/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *    20090126: alil, alil@users.sourceforge.net - fixed memeroy allocation bug identified by cppcheck
 *    20081001: alil, alil@users.sourceforge.net - redesigned baudrate assignment framework to be easierly adaptable on each 
 *                                                 architecture, fixed overflow bug in port assigment
 *******************************************************************************/
#include "txserhan.h"
#include "../../core/devexec.h"

#include "termios.h"
#include <unistd.h>
#include <sockapi.h>
#include <fcntl.h>

CSerialHandler &CSerialHandler::getInstance(void){
  static CTXSerialHandler oSerialHandler;
  return oSerialHandler;
}

int CTXSerialHandler::calcHighestSerialID(int pa_nSerialId, int pa_nMode){
  int nMaxID = 0;
  if(pa_nMode == 0){ //Adding a new SerialID
    if(pa_nSerialId > m_nHighestSerialID){
      m_nHighestSerialID = pa_nSerialId;
    }
  }
  else{ //Removing a SerialID
    std::list<SSerialHandlerListEntry>::iterator iter;
    for(iter = m_lSerialFBList.begin(); iter != m_lSerialFBList.end(); iter++){
      if((nMaxID < iter->m_nSerialID) && (iter->m_nSerialID != pa_nSerialId))
        nMaxID = iter->m_nSerialID;  
    }
    m_nHighestSerialID = nMaxID;
  }
  return 0;
}

bool CTXSerialHandler::run(void){
  m_eState = e_Run;
  selectRoutine();
  return true; //TODO correct return value
}

void CTXSerialHandler::selectRoutine(void){
  fd_set fdSerialReadSet;
  struct timeval stTimeOut;
  int nRetval;
  std::list<SSerialHandlerListEntry>::iterator iter;
  DEVLOG_INFO(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: select-thread started\n");
  
  while(m_bAlive){
    m_oSync.lock();
    stTimeOut.tv_sec = 0;
    stTimeOut.tv_usec = 250000;
    FD_ZERO(&fdSerialReadSet);
    for(iter = m_lSerialFBList.begin(); iter != m_lSerialFBList.end(); iter++){
			if(iter->m_bLocked == false){
        FD_SET(iter->m_nSerialID, &fdSerialReadSet);
			}
    }
    m_oSync.unlock();
    sel_mark:  
      nRetval = select(m_nHighestSerialID+1, &fdSerialReadSet, NULL, NULL, &stTimeOut);
    if(nRetval == -1){
      DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: select function failed\n");
      goto sel_mark; //dirty, but works!
    }
    if(nRetval != 0){
    //TODO: Check for time-out (nRetVal == 0). Also in socket handler??
      m_oSync.lock();
      DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: Checking which Serial ports received Data:\n");
      for(iter = m_lSerialFBList.begin(); iter != m_lSerialFBList.end(); iter++){
        if(FD_ISSET(iter->m_nSerialID, &fdSerialReadSet) != 0){
          iter->m_bLocked=true;
          checkReceiveData(*iter);
        }
      }
      m_oSync.unlock();  
    }
  }
}

int CTXSerialHandler::openSerialPort(CSerialHandler::SConnectionInformation &pa_stConnectionInformation){
  int nSerialID;
  unsigned int nPortNum;
  char acDevice[32];
  
  //Open a serial port
  nPortNum = extractPortNumber(pa_stConnectionInformation.m_pcConnectionID);
  if(nPortNum == 0){
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: extractPortNumber failed");
    return -1;
  }
  snprintf(acDevice, 32, "/com/%d", nPortNum-1);
  if((nSerialID = open(acDevice, O_RDWR)) == -1){
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: open failed");
    return -1;
  }
  calcHighestSerialID(nSerialID, 0);
  DEVLOG_INFO_3(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: Adding: %s with Baud rate %li\n", pa_stConnectionInformation.m_pcConnectionID, pa_stConnectionInformation.m_nBaudRate);
  //Set connection options
  if(setSerialPortOptions(nSerialID, pa_stConnectionInformation) != 0){
    DEVLOG_DEBUG_2(cg_nID_CExtEventHandler, 0x01, "CTXSerialHandler:: Failed setting port options for %i\n", nSerialID);
    return -1;
  }
  //Start thread if not running
  if (m_eState != e_Run) {
    start();
    m_eState = e_Run;
  }
  return nSerialID;
}

int CTXSerialHandler::closeSerialPort(SSerialHandlerListEntry &pa_stData){
  close(pa_stData.m_nSerialID);
  calcHighestSerialID(pa_stData.m_nSerialID, 1);
  pa_stData.m_nSerialID = -1;
  return 0;
}

int CTXSerialHandler::setSerialPortOptions(int pa_nSerialID, CSerialHandler::SConnectionInformation &pa_stConnectionInformation){
  //Get current port-options
  termios stPortOptions;
  tcgetattr(pa_nSerialID, &stPortOptions);
  //Set BaudRate (Input speeds sets output speed automatically)
  if(0 != cfsetispeed(&stPortOptions, 1200)){
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: Warning: Baud rate unknown. Set to default value (9600)\n");
    cfsetispeed(&stPortOptions, pa_stConnectionInformation.m_nBaudRate);
  }
  
  //Set other communication options
  stPortOptions.c_iflag = 0;
  stPortOptions.c_oflag = 0;
  stPortOptions.c_cflag |= CLOCAL|CREAD|CS8|CSIZE;
  stPortOptions.c_lflag = 0;
  //Set port-options
  return tcsetattr(pa_nSerialID, TCSANOW, &stPortOptions);
}

int CTXSerialHandler::receiveDataOnSerial(SSerialHandlerListEntry &pa_stData){
  int nRecvBytes = 0;
  unsigned int nToRecv = (pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nMaxSize) + 1;
  char *acRecvBuf = new char[(pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nMaxSize) + 1];
  int i = 0;
  bool bFirstLoop = true;
  double fTimeOut = 5.0;  //Timeout in seconds
  
  switch(pa_stData.m_stConnectionInformation.m_enServiceType){
    case e_SerComm:
      startStopWatch();
      while(nRecvBytes<nToRecv){
        nRecvBytes += read(pa_stData.m_nSerialID, &acRecvBuf[nRecvBytes], 1); //Read 1 byte at a time (most reliable)
        DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: Read a byte\n");
        if(nRecvBytes < 0)
          break;
        if(bFirstLoop){ //First loop
          nToRecv = (unsigned char)acRecvBuf[0];
          bFirstLoop = false;
        }
        if(nRecvBytes >= nToRecv){
          DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: Received all bytes, breaking read loop\n");
          break;
        }
        holdStopWatch();
        if(readStopWatch() > fTimeOut){
          DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"Error: read function generated a timeout\n");
          return -1;
        }
      }
      //Discard all data that is still present in the serial port buffers (in case of error in data transmission)
      tcflush(pa_stData.m_nSerialID, TCIOFLUSH);      
      //Copy data to send into send buffer (is there a function for this? strcat doesn't work!)
      for(i=0; i<(nRecvBytes-1); i++)
        pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_pcData[i] = acRecvBuf[i+1];
      pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize = nRecvBytes-1; 
      break;
    case e_Special:
      pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize = 
          read(pa_stData.m_nSerialID,
              pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_pcData,
              pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nMaxSize); //Always read maximum buffer size because it is unknown how much data there is on the serial port.
      break;
    default:
      DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: Wrong operation mode\n");
  }      
  
  if (pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize <= 0){
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: read failed\n");
    pa_stData.m_bLocked = false; //Retry next time again
    return(-1);
  }
  else{
    DEVLOG_INFO_2(cg_nID_CExtEventHandler, 0x01, "CTXSerialHandler:: receiveDataOnSerial %d bytes\n", pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize);
    pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_enConnectionState=e_ReceivedData;
  }
  //Informing SIFB if allowed
  if (sm_poDeviceExecution->extEvHandlerIsAllowed(m_nExtEvHandID)){ 
    DEVLOG_INFO(cg_nID_CExtEventHandler, 0x01, "CTXSerialHandler:: Starting EC\n");
    sm_poDeviceExecution->startNewEventChain(pa_stData.m_stConnectionInformation.m_poSerialFB);
  }
  else{
    //TODO: handle this !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    DEVLOG_DEBUG(cg_nID_CExtEventHandler, 0x01, "CTXSerialHandler:: Starting EC not allowed\n");
  }
  return 0;   
}

int CTXSerialHandler::sendDataOnSerial(SSerialHandlerListEntry &pa_stData) {
  int nToSend = 0;
  int nRetval = 0;
  char *acSendBuf;
  int i;
  
  switch(pa_stData.m_stConnectionInformation.m_enServiceType){
    case e_SerComm:
      nToSend = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_nFilledSize + 1;
      acSendBuf = new char[nToSend];
      acSendBuf[0] = nToSend;
      //Copy data to send into send buffer (is there a function for this? strcat doesn't work!)
      for(i=0; i<(nToSend-1); i++)
        acSendBuf[i+1] = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_pcData[i];
      break;
    case e_Special:  
      nToSend = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_nFilledSize;
      acSendBuf = new char[nToSend];
      //Copy data to send into send buffer (is there a function for this? strcat doesn't work!)
      for(i=0; i<(nToSend); i++)
        acSendBuf[i] = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_pcData[i];
      break;
    default:
      DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: Wrong operation mode\n");
  }

  DEVLOG_INFO_2(cg_nID_CExtEventHandler, 0x01, "CTXSerialHandler:: sending %d bytes\n", nToSend);
  tcflush(pa_stData.m_nSerialID, TCIOFLUSH);
  while(nToSend > 0){
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: sending data\n");
    nRetval = write(pa_stData.m_nSerialID, acSendBuf, nToSend);
    if(nRetval <= 0) //Write function failed
      return nRetval;
    nToSend -= nRetval;
    //FIXME: Line below is invalid, why? Needed if write function cannot write all data in one time.
//    acSendBuf += nRetval;
    tcdrain(pa_stData.m_nSerialID);
  }
    
  delete[] acSendBuf; // TODO think on how to remove this allocation/deallocation
  return 0; 
}

CTXSerialHandler::CTXSerialHandler(){
  m_nHighestSerialID=0;
  m_nNumSerial = 0;
  m_nExtEvHandID = sm_poDeviceExecution->registerExternalEventHandler(this);
  DEVLOG_INFO(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: started"); 
} ;

CTXSerialHandler::~CTXSerialHandler(){
//TODO: try to close all connections before exiting.
  DEVLOG_INFO(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: ended"); 
};

void CTXSerialHandler::startStopWatch(void){
  m_nStartTime = tx_time_get();
  m_bStartFlag = true;
  m_bHoldFlag = false;  
}

void CTXSerialHandler::holdStopWatch(){
  if(m_bStartFlag = true){
    m_nHoldTime = tx_time_get();
    m_bHoldFlag = true;
  }
  else{
    m_bHoldFlag = false;
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: Warning: setStopTime() called before setStartTime()\n");
  }
}

double CTXSerialHandler::readStopWatch(){
  if(m_bStartFlag && m_bHoldFlag)
    return (double)(m_nHoldTime - m_nStartTime) / (double)NABspTicksPerSecond;
  else{
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CTXSerialHandler:: Warning: readStopWatch(): return 0.0, for you must call 'startStopWatch()' and 'holdStopWatch()' first\n");
    return 0.0;
  }
}

void CTXSerialHandler::enableHandler(void) {
};  

void CTXSerialHandler::disableHandler(void) {
}; 

void CTXSerialHandler::setPriority(int pa_nPriority) {
};

int CTXSerialHandler::getPriority(void) {
};
