/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    20090126: alil, alil@users.sourceforge.net - fixed memeroy allocation bug identified by cppcheck
 *    20081001: alil, alil@users.sourceforge.net - redesigned baudrate assignment framework to be easierly adaptable on each 
 *                                                 architecture, fixed overflow bug in port assigment
 *******************************************************************************/
#include "pcserhan.h"
#include "../../core/devexec.h"
#include <fcntl.h>
#include <termios.h>

CSerialHandler &CSerialHandler::getInstance(void){
  static CPCSerialHandler oSerialHandler;
  return oSerialHandler;
}

const int CPCSerialHandler::scm_nBaudrateListSize = 19;
const CPCSerialHandler::SBaudrateAssignment CPCSerialHandler::scm_asBaudrateAssigment[] = {
    {1200, B1200}, {2400, B2400}, {4800, B4800}, {9600, B9600}, {19200, B19200}, {38400, B38400}, {57600, B57600}, {115200, B115200}, 
    {460800, B460800}, {500000, B500000}, {576000, B576000}, {921600, B921600}, {1000000, B1000000}, {1152000, B1152000}, 
    {1500000, B1500000}, {2000000, B2000000}, {2500000, B2500000}, {3000000, B3000000}, {3000000, B3000000}
    //These two are not defined on my machine: {3500000, B3500000}, {4000000, B4000000}
};



int CPCSerialHandler::calcHighestSerialID(int pa_nSerialId, int pa_nMode){
  int nMaxID = 0;
  if(pa_nMode == 0){ //Adding a new SerialID
    if(pa_nSerialId > m_nHighestSerialID){
      m_nHighestSerialID = pa_nSerialId;
    }
  }
  else{ //Removing a SerialID
    std::list<SSerialHandlerListEntry>::iterator iter;
    for(iter = m_lSerialFBList.begin(); iter != m_lSerialFBList.end(); iter++){
      if((nMaxID < iter->m_nSerialID) && (iter->m_nSerialID != pa_nSerialId))
        nMaxID = iter->m_nSerialID;  
    }
    m_nHighestSerialID = nMaxID;
  }
  return 0;
}

void CPCSerialHandler::selectRoutine(void){
  fd_set fdSerialReadSet;
  struct timeval stTimeOut;
  int nRetval;
  std::list<SSerialHandlerListEntry>::iterator iter;
  DEVLOG_INFO(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: select-thread started\n");

  while(m_bAlive){
    m_oSync.lock();
    stTimeOut.tv_sec = 0;
    stTimeOut.tv_usec = 250000;
    FD_ZERO(&fdSerialReadSet);
    for(iter = m_lSerialFBList.begin(); iter != m_lSerialFBList.end(); iter++){
			if(iter->m_bLocked == false){
        FD_SET(iter->m_nSerialID, &fdSerialReadSet);
			}
    }
    m_oSync.unlock();
    sel_mark:  
      nRetval = select(m_nHighestSerialID+1, &fdSerialReadSet, NULL, NULL, &stTimeOut);
    if(nRetval == -1){
      DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: select function failed\n");
      goto sel_mark; //dirty, but works!
    }
    if(nRetval != 0){
    //TODO: Check for time-out (nRetVal == 0). Also in socket handler??
      m_oSync.lock();         
      DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: Checking which Serial ports received Data:\n");
      for(iter = m_lSerialFBList.begin(); iter != m_lSerialFBList.end(); iter++){
        if(FD_ISSET(iter->m_nSerialID, &fdSerialReadSet) != 0){
          iter->m_bLocked=true;
          checkReceiveData(*iter);
        }
      }
      m_oSync.unlock();  
    }
  }
}

int CPCSerialHandler::openSerialPort(CSerialHandler::SConnectionInformation &pa_stConnectionInformation){
  int nSerialID;
  unsigned int nPortNum;
  char acDevice[32];
  
  //Open a serial port
  nPortNum = extractPortNumber(pa_stConnectionInformation.m_pcConnectionID);
  if(nPortNum == 0){
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: extractPortNumber failed");
    return -1;
  }
  snprintf(acDevice, 32, "/dev/ttyS%d", nPortNum-1);
  if((nSerialID = open(acDevice, O_RDWR)) == -1){
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: open failed");
    return -1;
  }
  calcHighestSerialID(nSerialID, 0);
  DEVLOG_INFO_3(cg_nID_CExtEventHandler, 0x01, "CPCSerialHandler:: Adding: %s with Baud rate %li\n", pa_stConnectionInformation.m_pcConnectionID, pa_stConnectionInformation.m_nBaudRate);
  //Set connection options
  if(setSerialPortOptions(nSerialID, pa_stConnectionInformation) != 0){
    DEVLOG_DEBUG_2(cg_nID_CExtEventHandler, 0x01, "CPCSerialHandler:: Failed setting port options for %i\n", nSerialID);
    return -1;
  }
  //Start thread if not running
  if (m_eState != e_Run) {
    start();
    m_eState = e_Run;
  }
  return nSerialID;
}

int CPCSerialHandler::closeSerialPort(SSerialHandlerListEntry &pa_stData){
  close(pa_stData.m_nSerialID);
  calcHighestSerialID(pa_stData.m_nSerialID, 1);
  pa_stData.m_nSerialID = -1;
  return 0;
}

int CPCSerialHandler::setSerialPortOptions(int pa_nSerialID, CSerialHandler::SConnectionInformation &pa_stConnectionInformation){
  //Get current port-options
  termios stPortOptions;
  tcgetattr(pa_nSerialID, &stPortOptions);
  //Set BaudRate
  int i;
  for(i = 0; i < scm_nBaudrateListSize; ++i){
    if(scm_asBaudrateAssigment[i].m_nBaudrate == pa_stConnectionInformation.m_nBaudRate){
      cfsetispeed(&stPortOptions, scm_asBaudrateAssigment[i].m_nParam);
      cfsetospeed(&stPortOptions, scm_asBaudrateAssigment[i].m_nParam);
      i = -1;
      break;
    }
  }
  
  if(-1 != i){
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: Warning: Baud rate unknown. Set to default value (9600)\n");
    cfsetispeed(&stPortOptions, B9600);
    cfsetospeed(&stPortOptions, B9600);
  }
  
  //Set other communication options
  stPortOptions.c_iflag = 0;
  stPortOptions.c_oflag = 0;
  stPortOptions.c_cflag |= CLOCAL|CREAD|CS8|CSIZE;
  stPortOptions.c_lflag = 0;
  //Set port-options
  return tcsetattr(pa_nSerialID, TCSANOW, &stPortOptions);
}

int CPCSerialHandler::receiveDataOnSerial(SSerialHandlerListEntry &pa_stData){
  unsigned int nRecvBytes = 0;
  unsigned int nToRecv = (pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nMaxSize) + 1;
  char *acRecvBuf = new char[(pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nMaxSize) + 1];
  int i = 0;
  bool bFirstLoop = true;
  double fTimeOut = 5.0;  //Timeout in seconds
  
  switch(pa_stData.m_stConnectionInformation.m_enServiceType){
    case e_SerComm:
      startStopWatch();
      while(nRecvBytes < nToRecv){
        if(read(pa_stData.m_nSerialID, &acRecvBuf[nRecvBytes], 1) < 0) //Read 1 byte at a time (most reliable)
          break;
        ++nRecvBytes;
        DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: Read a byte\n");                
        if(bFirstLoop){ //First loop
          nToRecv = (unsigned char)acRecvBuf[0];
          bFirstLoop = false;
        }
        if(nRecvBytes == nToRecv){
          DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: Received all bytes, breaking read loop\n");
          break;
        }
        holdStopWatch();
        if(readStopWatch() > fTimeOut){
          DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: Error: read function generated a timeout\n");
          return -1;
        }
      }  
      //Discard all data that is still present in the serial port buffers (in case of error in data transmission)
      tcflush(pa_stData.m_nSerialID, TCIOFLUSH);          
      //Copy data to send into send buffer (is there a function for this? strcat doesn't work!)
      for(i = 0; i < (nRecvBytes-1); ++i){
        pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_pcData[i] = acRecvBuf[i+1];
      }  
      pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize = nRecvBytes-1;
      break;
    case e_Special:
      pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize = 
          read(pa_stData.m_nSerialID,
              pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_pcData,
              pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nMaxSize); //Always read maximum buffer size because it is unknown how much data there is on the serial port.
      break;
    default:
      DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: Wrong operation mode\n");
  }
  
  if (pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize <= 0){
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: read failed\n");
    pa_stData.m_bLocked = false; //Retry next time again
    return -1;
  }
  else{
    DEVLOG_INFO_2(cg_nID_CExtEventHandler, 0x01, "CPCSerialHandler:: receiveDataOnSerial %d bytes\n", pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize);
    pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_enConnectionState=e_ReceivedData;
  }
  //Informing SIFB if allowed
  if (sm_poDeviceExecution->extEvHandlerIsAllowed(m_nExtEvHandID)){ 
    DEVLOG_INFO(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: Starting EC\n");
    sm_poDeviceExecution->startNewEventChain(pa_stData.m_stConnectionInformation.m_poSerialFB);
  }
  else{
    //TODO: handle this !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: Starting EC not allowed\n");
  }
  return 0;   
}

int CPCSerialHandler::sendDataOnSerial(SSerialHandlerListEntry &pa_stData) {
  int nToSend = 0;
  int nRetval = 0;
  char *acSendBuf;
  int i;
  
  switch(pa_stData.m_stConnectionInformation.m_enServiceType){
    case e_SerComm:
      nToSend = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_nFilledSize + 1;
      acSendBuf = new char[nToSend];
      acSendBuf[0] = nToSend;
      //Copy data to send into send buffer (is there a function for this? strcat doesn't work!)
      for(i=0; i<(nToSend-1); i++)
        acSendBuf[i+1] = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_pcData[i];
      break;
    case e_Special:
      nToSend = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_nFilledSize;
      acSendBuf = new char[nToSend];
      //Copy data to send into send buffer (is there a function for this? strcat doesn't work!)
      for(i=0; i<(nToSend); i++)
        acSendBuf[i] = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_pcData[i];
      break;
    default:
      DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: Wrong operation mode\n");
  }
  
  DEVLOG_INFO_2(cg_nID_CExtEventHandler, 0x01,"CPCSerialHandler:: sending %d bytes\n", nToSend);
  tcflush(pa_stData.m_nSerialID, TCIOFLUSH);
  while(nToSend > 0){
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: sending data\n");
    nRetval = write(pa_stData.m_nSerialID, acSendBuf, nToSend);
    if(nRetval <= 0) //Write function failed
      return nRetval;
    nToSend -= nRetval;
    //FIXME: Line below is invalid, why? Needed if write function cannot write all data in one time.
//    acSendBuf += nRetval;
    tcdrain(pa_stData.m_nSerialID);
  }
    
  delete[] acSendBuf;  // TODO think on how to remove this allocation/deallocation
  return 0;
}

CPCSerialHandler::CPCSerialHandler(){
  m_nHighestSerialID=0;
  m_nNumSerial = 0;
  m_nExtEvHandID = sm_poDeviceExecution->registerExternalEventHandler(this);
  DEVLOG_INFO(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: started"); 
} ;

CPCSerialHandler::~CPCSerialHandler(){
  //TODO: try to close all connections before exiting.
  DEVLOG_INFO(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: ended"); 
};

void CPCSerialHandler::startStopWatch(void){
  gettimeofday(&m_stStartTime, 0);
  m_bStartFlag = true;
  m_bHoldFlag = false;  
}

void CPCSerialHandler::holdStopWatch(){
  if(m_bStartFlag == true){
    gettimeofday(&m_stHoldTime, 0);
    m_bHoldFlag = true;
  }
  else{
    m_bHoldFlag = false;
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: Warning: setStopTime() called before setStartTime()\n");
  }
}

double CPCSerialHandler::readStopWatch(){
  if(m_bStartFlag && m_bHoldFlag)
    return (m_stHoldTime.tv_sec-m_stStartTime.tv_sec
      +(double(m_stHoldTime.tv_usec-m_stStartTime.tv_usec)/1e+6));
  else{
    DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CPCSerialHandler:: Warning: readStopWatch(): return 0.0, for you must call 'startStopWatch()' and 'holdStopWatch()' first\n");
    return 0.0;
  }
}

void CPCSerialHandler::enableHandler(void) {
};  

void CPCSerialHandler::disableHandler(void) {
}; 

void CPCSerialHandler::setPriority(int ) {
};

int CPCSerialHandler::getPriority(void) {
  return 1;    
};
