/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _THREAD_H_
#define _THREAD_H_

#include <semaphore.h>
#include <pthread.h>
#include "../datatype.h"

#define CThread CPCThread  //allows that doxygen can generate better documenation

/**  \ingroup FORTE-HAL 
 * \defgroup pc_hal PC FORTE Hardware Abstraction Layer
 *
 * \brief The FORTE-HAL implemenation for normal PCs
 *
 * It will mainly work under linux or linux similar environments like cygwin.
 */

/*! \ingroup pc_hal 
 * \brief The thread implementation for the posix thread interface. 
 *  
 * @author AZ
 * @version 20051009/AZ - Initial implementation
 */
class CPCThread{
  private:   
  TUINT32 m_nDeadline; //!deadline the thread needs to be finish its exectuion. 0 means unconstraint. 
  
  sem_t s;

/*! \brief data needed for posix scheduling system to identify the thread.
 */  
   pthread_t m_stThreadID;
/*! \brief Size of the stack used by this thread.
 */  
   long m_nStackSize;

/*! \brief Flag indicating if the Thread is suspended
 * 
 * This flag my not be neccessary if the underling system has an easy way to retrive this information.
 * Otherwise such a flag can result in faster code.
 */    
    bool m_bSuspended;
 
/*! \brief Creates a new thread.
 * 
 *  With this function all the setup things for a new thread are done. The Thread created is initialy 
 *  suspended and with the start() method the execution of the code in the run() method is started.
 *  @param pa_nStackSize the Size of the stack the thread is allowed to use. 0 means use system default stack size.
 */   
   bool create(long pa_nStackSize = 0);

/*! \brief Destroies the thread
 * 
 *  This function destroies all the data structures created in the creation phase. All used memory is freed.
 */     
   bool destroy(void);  
protected:
/*! \brief Flag that indicates if the Thread is alive.
 * 
 *  This flag has two main purposes:
 *    -# indicate for other classes if the thread is still executing
 *    -# use in the run()-method to check if the thread is still allowed to execute (e.g. while(isAlive()) ). 
 *       This is important for stopping and destroying threads.
 */   
    bool m_bAlive;    
/*!\brief Funciton that is given to the system thread support that should be called for the thread.
 * 
 * this function will call the run method of the thread instance.
 */    
    static void * threadFunction(void *arguments); 
/*! \brief Suspends the thread.
 * 
 *  Suspends the execution of the thread until resumeSelfSuspend(), end(), or join() is called.
 */     
    void selfSuspend(void);
    
  public:
/*! \brief Constructor of the Thread class
 *  
 *  Does all the neccessary steps in order to get the thread running with the start()-method   
 *  @param pa_nStackSize the Size of the stack the thread is allowed to use. 0 means use system default stack size.
 */     
   CPCThread(long pa_nStackSize = 0);

/*! \brief Stops and destroys thread.
 * 
 *  Will stop the execution if running and destroy the thread including all system specific data.
 */     
   virtual ~CPCThread();
   
/*! \brief Indicates if the thread is allowed to execute.
 * 
 *  This functions checks if the Thread is still executing user code in its run()-method.
 *  \return true if there the run method is acitve.
 */     
   bool isAlive(void);   

  void setDeadline(TUINT32 pa_nVal); //!Set the deadline of the thread. 
  TUINT32 getDeadline(void); //!Get the current deadline of the thread.
    
/*! \brief Abstract method for the code to execute in the thread.
 * 
 *  This thread class has to provide means that the code a inheriting class will add to the run()-method will
 *  be executed in a seperated thread regarding the creator of the CThread class.
 * 
 *  The inheriting class has to fullfill the folloing rules when using the run method:
 *    - To end the thread execution simple leave the run()-method 
 *    - In order to allow the deletion and stopping of the thread add frequent checks to isAlive and end the
 *      execution if isAlive() returns false.
 * 
 *  \return true if thread execution ended successfull
 */   
   virtual bool run() = 0;

/*! \brief starts the Thread
 * 
 *  By calling this method the execution in the run()-Method will be started. If necessary additional data
 *  can be created here. Because of inheritance resons the best place for executing create is in this method.
 *  For details ask AZ.
 */     
   void start(void);

/*! \brief Resumes a suspended Thread
 * 
 *  
 */     
   void resumeSelfSuspend(void);

/*! \brief Stops the execution of the thread
 * 
 *  This function imidiatly stops the execution of the thread (seting m_bAlive to false) and waits till 
 *  this is finished.
 */     
   void end(void);

/*! \brief Waits for the Thread to finish its execution.
 * 
 *  This function waits till the execution in the thread decides to end the execution. Blocks the caller!!!
 */     
   void join(void);       // Waits 
};

#endif /*THREAD_H_*/


