/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *    20081001: alil, alil@users.sourceforge.net - redesigned baudrate assignment framework to be easierly adaptable on each architecture
 *******************************************************************************/ 
#ifndef _CSERIALHANDLER_H_
#define _CSERIALHANDLER_H_

#include <list>
#include "../core/extevhan.h"
#include "../core/esfb.h"
#include "active_arch/sync.h"

/*!\brief This class is a wrapper class for the architecture or system specific serial interface functions.
 * \ingroup EXTEVHAND 
 *   
 * @author jbrunn
 * @version 20061214/jbrunn - Changed check if FB already registered with SH
 * @version 20061204/jbrunn - Changed service types
 * @version 20061201/jbrunn - Added stopwatch functions
 * @version 20061130/jbrunn - Replaced old Serial Handler with this one
 * @version 20061128/jbrunn - Inherited Serial Handler from External Event Handler
 * @version 20061122/jbrunn - First working version
 * @version 20061115/jbrunn - Initial version
 */
class CSerialHandler : public CExternalEventHandler{  
  public:
    //!Lists the possible service types.
    enum ESerialServiceType{e_NoService,
                            e_SerComm,     //!<Communication mode, checks if number of received bytes equals number of sent bytes
                            e_Special,    //!<Special mode, just sends and receives the data
                            };
    //!Lists the possible service states.
    enum ESerialHandlerState{e_Init, e_Stop, e_Run, e_Error};
    //!Lists the possible connection states.
    enum EConnectionState{e_Unconnected, e_ReceivedData};

    //!Holds data and properties for sending and receiving data.
    struct SSharedData {
        char *m_pcData; //!<Data to be received or sent.
        int m_nMaxSize; //!<Maximum data size.
        int m_nFilledSize; //!<Used data size.
        EConnectionState m_enConnectionState; //!<State of the connection.
    };

    //!Holds information about the FB that calls the serial service.
    struct SConnectionInformation {
        CEventSourceFB *m_poSerialFB; //!<Event source FB that requests the serial service.
        ESerialServiceType m_enServiceType; //!<Service type.
        char m_pcConnectionID[8]; //!<Connection ID (COMn, where n = 1,2,...,9).
        unsigned long m_nBaudRate; //!<Baud rate of the connection.
        SSharedData *m_pstSendBuffer; //!<Buffer structure that holds the necessary information to send data.
        SSharedData *m_pstRecvBuffer; //!<Buffer structure that holds the necessary information to receive data.
    };
  	  virtual ~CSerialHandler(){};
  //! Gets the address of the Serial Handler object. Will be implemented by the according architecture.
    static CSerialHandler &getInstance(void);
  /*!\brief Connect a SIFB to the Serial Handler.
   * 
   * \param pa_stConnectionInformation Connection information of the FB
   * \return File handler of the port or -1 on error
   */
    int registerSerialFB(SConnectionInformation &pa_stConnectionInformation); 
  /*!\brief Disconect a SIFB from the Serial Handler
   * 
   * \param pa_poSerialFB FB that wants to be unregistered
   * \return Status value
   */
    int unregisterSerialFB(CEventSourceFB *pa_poSerialFB);
  /*!\brief Sends Data to a Serial port via SHM
   *
   * \param pa_poSerialFB The FB that is sending
   * \return Status value
   */
    int sendData(CEventSourceFB *pa_poSerialFB);
  /*!\brief Called when a FB has read the SHM
   *
   * \param pa_poSerialFB The FB that is sending
   * \return Status value
   */
    int receivedData(CEventSourceFB *pa_poSerialFB);
  private:
  protected:
  //!ID of the External Event Handler that the Serial Handler is registered to.
    int m_nExtEvHandID;
  //!Link to an inter-thread synchronization object.
    CSyncObject m_oSync;
  //! Variable representing the internal state of the Serial Handler.
    ESerialHandlerState m_eState;
  //! Data stored for each FB that is registered to the Serial Handler.  
    struct SSerialHandlerListEntry{
      SConnectionInformation m_stConnectionInformation; //!<TConnectionInformation structure.
      int m_nSerialID; //!<File handler of the opened serial port.
      bool m_bLocked; //!<Indicates if the receiver is locked. If false then the receiver can grab data from the serial port.
    }; 
  //!List of all function blocks currently registered to the Serial Handler.  
    list<SSerialHandlerListEntry> m_lSerialFBList;
  //!We don't like to have a direct instance of this class, so the default constructor is protected.
    CSerialHandler(void){
      m_eState = e_Init;  
    };
  /*!\brief Function that is called by the implementation of a Serial Handler. It will be called 
   * when data is received on the specific Serial Port.
   *
   * \param pa_stSerialListEntry FB that wants to receive data
   * \return Status value 
   */    
    int checkReceiveData(SSerialHandlerListEntry &pa_stSerialListEntry);    
  /*!brief Gets the port number from the port description
   * 
   * \param pa_Port The character array that contains the port description
   * \return Port number (range 1-9), or 0 on error
   */
    unsigned int extractPortNumber(const char *pa_Port);
  /*!\brief Opens a serial port to send and receive data.
   * 
   * \param pa_stConnectionInformation Connection information structure
   * \return File handler of the port or -1 on error
   */
    virtual int openSerialPort(SConnectionInformation &pa_stConnectionInformation) = 0;
  /*!\brief Closes a serial port.
   * 
   * \param pa_stData FB that requests to close a serial port.
   * \return Status value
   */ 
    virtual int closeSerialPort(SSerialHandlerListEntry &pa_stData) = 0; 
  /*!\brief Sets connection options for a serial port.
   * 
   * \param pa_nSerialID File handler of the serial port.
   * \param pa_stConnectionInformation Connection information structure.
   * \return Status value
   */ 
    virtual int setSerialPortOptions(int pa_nSerialID, SConnectionInformation &pa_stConnectionInformation) = 0; 
  /*!\brief Sends data to a serial port.
   *
   * \param pa_stData Structure that holds the data to send.
   * \return Status Value
   */
    virtual int sendDataOnSerial(SSerialHandlerListEntry &pa_stData) = 0;
  /*!\brief Receives data from a serial port.
   *
   * \param pa_stData Structure that holds the read buffer.
   * \return Status Value
   */
    virtual int receiveDataOnSerial(SSerialHandlerListEntry &pa_stData) = 0;
  /*!\brief Sets the start time of a stopwatch to the current time.
   * 
   * The stopwatch is used to break the read loop after a certain time.
   * This is necessary because a send error could occur that makes the
   * read loop an infinite loop.
   * The kind of timer is not important. This can e.g. be the system clock. 
   */   
    virtual void startStopWatch(void) = 0;
  /*!\brief Sets the hold time of a stopwatch to the current time.
   * 
   * If the start function has not been called this function does nothing. 
   */   
    virtual void holdStopWatch(void) = 0;
  /*!\brief Gets the elapsed time between the start and hold time in seconds.
   * 
   * If the start and hold functions not have been called this function returns 0.0.
   * \retval The elapsed time between the start and hold time or 0.0 on error.
   */   
    virtual double readStopWatch(void) = 0;
};

#endif /*_CSERIALHANDLER_H_*/
