/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/ 
#ifndef _CSOCKETHANDLER_H_
#define _CSOCKETHANDLER_H_

#include <list>
#include "../core/extevhan.h"
#include "../core/esfb.h"
#include "active_arch/sync.h"

/**\brief Base class for ethernet communication.
 *  \ingroup EXTEVHAND
 *
 * @author smr
 * @version 20070911/AZ - Removed local comm definitions from ESocketServiceType
*/
class CSocketHandler : public CExternalEventHandler{
  public:
    enum ESocketServiceType{e_NoService, e_Server, e_Client, e_Publisher, e_Subscriber};
    enum ESocketHandlerState{e_Init, e_Stop, e_Run, e_Error};
    enum EConnectionState{e_Unconnected, e_ReceivedData, e_ClosedByClient};

    struct SSharedData {
        TBYTE *m_pcData;
        int m_nMaxSize;
        int m_nFilledSize;
        EConnectionState m_enConnectionState;
    };

    struct SConnectionInformation{
        CEventSourceFB *m_poSocketFB;
        ESocketServiceType m_enServiceType;
        char m_pcConnectionID[32];
        SSharedData *m_pstSendBuffer;
        SSharedData *m_pstRecvBuffer;    
    };
  private:
  protected:
    int m_nExtEvHandID;  
    
    CSyncObject m_oSync;
   //! Variable representing the internal state of the socket handler
    ESocketHandlerState m_eState;
   //! Data stored for each FB that is registered to the socket handler  
    struct SSocketHandlerListEntry{
      SConnectionInformation m_stConnectionInformation;
      int m_nSocketID;
      int m_nAcceptID;
      bool m_bAcceptedConnection;
      bool m_bLocked;
    }; 
   //! List of all function blocks currently registered to the socket handler  
    list<SSocketHandlerListEntry> m_lSocketFBList;
    
   //! we don't like to have a direct instance of this class    
    CSocketHandler(void){
      m_eState = e_Init;
    }; 
  /*!\brief Function that is called by the implementation of an socket handler it will be called 
   * when data is received for the specific socket
   *
   * \param SocketID, where Data is ready to receive
   * \return Status Value 
   */    
    int checkReceiveData(SSocketHandlerListEntry &pa_stSocketListEntry);
  /*!\brief This function is used to split a string in the form "x.x.x.x:port" into IP-Adress and Port
   * 
   * \param pointer to string to strip
   * \return pointer to string with port
   */
    char *splitString(char *pa_psIn);  
    
   /*!\brief Open an new TCP connection
   * 
   *  \return socketid or -1 on error
   */ 
    virtual int openTCPConnection(SConnectionInformation &pa_stConnectionInformation) = 0;
    virtual int openUDPSendPort(SConnectionInformation &pa_stConnectionInformation) = 0;
    virtual int openUDPRecvPort(SConnectionInformation &pa_stConnectionInformation) = 0;
    
    virtual int closeTCPConnection(SSocketHandlerListEntry &pa_stData) = 0;
    virtual int closeUDPPort(SSocketHandlerListEntry &pa_stData) = 0;
    
   /*!\brief Sends Data on an Connection
     *
     * \param Pointer to Data Structure
     * \return Status Value
     */
    virtual int sendDataOnTCP(SSocketHandlerListEntry &pa_stData) = 0;
    virtual int sendDataOnUDP(SSocketHandlerListEntry &pa_stData) = 0;
  /*!\brief Accepts an incomming Connection (for Client/Server only)
   *
   * \param Reference to Data Structure
   * \return Status Value
   */
    virtual int acceptTCPConnection(SSocketHandlerListEntry &pa_stData) = 0;

  /*!\brief Receives Data on an Connection
   *
   * \param Reference to Data Structure
   * \return Status Value
   */
    virtual int receiveDataOnTCP(SSocketHandlerListEntry &pa_stData) = 0;
    virtual int receiveDataOnUDP(SSocketHandlerListEntry &pa_stData) = 0;
    
    
  public:	 
	 virtual ~CSocketHandler(){};
   
   static CSocketHandler &getInstance(void); //! will be implemented by the according architecture
   
   
  /*!\brief Connect a SIFB to the socket handler
   * 
   *  \return socketid or -1 on error
   */
  int registerSocketFB(SConnectionInformation &pa_stConnectionInformation); 

  /*!\brief Disconect a SIFB to the socket handler
   */
  int unregisterSocketFB(CEventSourceFB *pa_poSocketFB);
  
  /*!\brief Sends Data on a Socket via SHM
   *
   * \param pa_poSocketFB The FB that is sending
   * \return Status Value
   */
  int sendData(CEventSourceFB *pa_poSocketFB);

  /*!\brief Called when a FB has read the SHM
   *
   * \param pa_poSocketFB The FB that is sending
   * \return Status Value
   */
  virtual int receivedData(CEventSourceFB *pa_poSocketFB);
  
  /*!\brief Called when TCP connection was closed by client
   * 
   * This function closes the accept-ports 
   * and listens to the listener-socket of the server again
   * 
   * \param pa_poSocketFB The FB that is calling
   * \param pa_stConnectionInformation Connection Information (e.g. Address, Port)
   * \return socketid or -1 on error
   */
  // virtual int reopenSocketFB(CEventSourceFB *pa_poSocketFB,TConnectionInformation &pa_stConnectionInformation);
   
};

#endif /*_CSOCKETHANDLER_H_*/
