/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20071029: az, alil@users.sourceforge.net - destrucutor now deletes the m_poValue
 *******************************************************************************/
#ifndef _DATACONN_H_
#define _DATACONN_H_

#include "./datatypes/any.h"
#include "conn.h"

/*! \ingroup CORE\brief Class for handling a data connection.
 *  @author AZ
 *  @version 20081121/mmm - removed readDataWithCasting
 *  @version 20081118/mmm - changed getValue, setValue and connect to eliminate ANY_FAC
 *  @version 20071029/az - destrucutor now deletes the m_poValue
 *  @version 20070911/az - Fixed bug in the rollback mechanism of connect data connections if a connection already exists
 *  @version 20060718/az - reworked connect method
 *  @version 20060717/gugr & tstras - setValue() & getValue() method added
 *  @version 20060214/az - fixed some bugs in the connect() regarding the cloning for the m_poValue
 *  @version 20060208/gugr - changed connect() for better readability & validity checks
 *  @version 20060124/smr - in connect(): fixed getDataOutput / getDataInput mismatch
 *  @version 20051202/tstras - member m_poValue, writeData() & readData() added
 *  @version 20050920/AZ - Initial version as basis for the interface documentation
 */
class CDataConnection : public CConnection {
  private:
  protected:
 /*! \brief Value for storing the current data of the connection
 */
    CIEC_ANY *m_poValue;

		bool m_bCasting; // At least one of the managed links goes to another type

  public:
    CDataConnection(TConnectionID pa_nSourceId):
      CConnection(pa_nSourceId) {
      m_poValue = 0;
      m_bCasting = false;
    };
    CDataConnection(const char *pa_acFBName, const char *pa_acOutputName):
      CConnection(CConnection::genConPortId(pa_acFBName, pa_acOutputName)) {
      m_poValue = 0;
      m_bCasting = false;
    };
    virtual ~CDataConnection(){
      delete m_poValue;
    };
/*! \brief Establishes the connection.
 *
 *   With this command the connection is created and the FBs are set to appropriate states.
 *   This mainly effects the destination of the connection (because of the fan-out).
 *   If the referenced connection through the pa_sDst parameter is the first destination
 *   also the src reference is created as necessary.
 *   \param pa_nDstId   String of the destination of the connection
 *   \param pa_roSrcFB  Reference to the source FB of this connection.
 *   \param pa_roDstFB  Reference to the destination FB of this connection.
 *   \param pa_sDstInput Name of the input of the destination FB.
 *   \return Can be the following response:
 *     - e_RDY ... creation went well.
 *     - e_NO_SUCH_OBJECT ... The destination is not a valid input.
 *     - e_INVALID_STATE ... The specified connection already exists.
 */
    virtual EMGMResponse connect(TConnectionID pa_nDstId, CFunctionBlock &pa_roSrcFB, CFunctionBlock &pa_roDstFB, const char *pa_sDstInput);

    virtual EMGMResponse connect(const char *pa_sDstFB, CFunctionBlock &pa_roSrcFB, CFunctionBlock &pa_roDstFB, const char *pa_sDstInput){
  return connect(CConnection::genConPortId(pa_sDstFB, pa_sDstInput),
                 pa_roSrcFB, pa_roDstFB, pa_sDstInput);
};
/*! \brief Disconnects the connection.
 *
 *   With this command the connection is removed and the FBs are set to appropriate states.
 *   This mainly effects the destination of the connection (because of the fan-out).
 *   If the referenced connection through the pa_sDst parameter is the last destination
 *   also the src reference is removed.
 *   \param pa_nDstId     String of the destination of the connection
 *   \param pa_roSrcFB  Reference to the source FB of this connection.
 *   \param pa_roDstFB  Reference to the destination FB of this connection.
 *   \param pa_sDstInput Name of the input of the destination FB.
 *   \return Can be the following response:
 *     - e_RDY ... Deletion went well.
 *     - e_NO_SUCH_OBJECT ... The destination is not a valid input.
 */
    virtual EMGMResponse disconnect(TConnectionID pa_nDstId, CFunctionBlock &pa_roSrcFB, CFunctionBlock &pa_roDstFB, const char *pa_sDstInput);
/*! \brief Write connection data value.
 *
 *   Write data value from FB data output to connection data variable.
 *   \param pa_poValue pointer to FB data output
 *   \return Can be the following response:
 *     - TRUE ... write successful
 *     - FALSE ... no such data connection exists
 */
    void writeData(CIEC_ANY *pa_poValue){
	    if(m_poValue){
	      m_poValue->setValue(*pa_poValue);
		  }
    };
/*! \brief Read connection data value.
 *
 *   Read data value from connection data variable to FB data input.
 *   \param pa_poValue pointer to FB data input
 *   \return Can be the following response:
 *     - TRUE ... read successful
 *     - FALSE ... no such data connection exists
 */
    void readData(CIEC_ANY *pa_poValue) {
			if(m_poValue) {
			  pa_poValue->setValue(*m_poValue);
			}
    }

/*! \brief Set class member variable m_poValue.
 *
 *   Set class member variable m_poValue.
 *   \param pa_poValue ... pointer to data variable
 */
    void setValue(CIEC_ANY *pa_poValue) {
      m_poValue = pa_poValue;
    }
/*! \brief Get class member variable m_poValue.
 *
 *   Get class member variable m_poValue.
 *   \return pointer to class member variable m_poValue
 */
    CIEC_ANY* getValue() {
      return m_poValue;
    }
};

typedef CDataConnection *TDataConnectionPtr;

#endif /*_DATACONN_H_*/
