/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _ANY_H_
#define _ANY_H_

#include <string>
#include "../typelib.h"

#define FORTE_USE_FLOAT_DATATYPES //!< if #define FORTE_USE_FLOAT_DATATYPES is set, FORTE will use FLOAT dataytypes, if not Float and Double datatypes will not be available
#define FORTE_USE_64BIT_DATATYPES //!< if #define FORTE_USE_64BIT_DATATYPES is set, FORTE will datatypes up to 64 bit, if not then only 32 bit datatypes will be used
//TODO develop a better way for setting the endianess
#define FORTE_LITTLE_ENDIAN //!< Must be set to FORTE_LITTLE_ENDIAN or FORTE_BIG_ENDIAN, if not Endianess not set! error will be shown

using namespace std;

/*!\ingroup COREDTS  CIEC_ANY represents the IEC_ANY data type according to IEC 61131.
 *  @author tstras & imuell
 *  @version 20081216/mmm -removed serialize, deserialize, serializeshort, deserializeshort, serializeValue,
 *  deserializeValue, serializeTag and deserializeTag
 *  @version 20081121/mmm - added EDataTypeTags, rearranged EDataTypes, removed getValue(), made setValue() public,
 *  added setValueComplex for complex datatypes, added set and get functions for simple datatypes, modified setValue,
 *  added union UAnyData for simple datatypes, removed castFrom, serialize function don't has to be virtual
 *  @version 20081121/mmm - removed getValue, getValueComplex and castFrom
 *  @version 20081112/mmm - added m_eDataTypeId for datatype identification and
 *              changed data representation to union; implementing corresponding setValue and
 *              getValue methods, constructor
 *  @version 20070307/az - changed serialization code for supporting also the
 *                         Serialization of arrays.
 *  @version 20061107/gebenh - moved serializedouble() and
 *                             deserializedouble() to CIEC_LREAL
 *  @version 20060322/tstras - isFacadeDataType() method added
 *  @version 20051205/tstras - clone() method added
 *  @version 20051201/tstras - getDataTypeID() method, setValue(IIEC_ANY*) &
 *                             getValue(IIEC_ANY*) added
 *  @version 20051017/tstras & imuell - Initial version
 */

/*!\ingroup CORE \defgroup COREDTS Core Data Types
 * \brief Implementation of the core IEC 61131-3 data types.
 *
 */

class CIEC_ANY {
public:

  /*! \ingroup COREDTS\brief Data type ID's are chosen according to IEC 61499-1 : Function Block
   *  -- Part 1 Architecture - Annex F (informative) Information exchange/F.3
   *  Transfer syntaxes (page 89).
   */
  enum EDataTypeID {
    e_ANY,
    e_BOOL,
    e_SINT,
    e_INT,
    e_DINT,
    e_LINT,
    e_USINT,
    e_UINT,
    e_UDINT,
    e_ULINT,
    e_BYTE,
    e_WORD,
    e_DWORD,
    e_LWORD,
    e_REAL,
    e_LREAL,
    e_DATE,
    e_TIME_OF_DAY,
    e_DATE_AND_TIME,
    e_TIME, //until here simple Datatypes
    e_STRING,
    e_WSTRING,
    e_DerivedData,
    e_DirectlyDerivedData,
    e_EnumeratedData,
    e_SubrangeData,
    e_ARRAY,  //according to the compliance profile
    e_STRUCT
  };

  CIEC_ANY(const EDataTypeID pa_eDataTypeID) { m_eDataTypeId = pa_eDataTypeID;
  #ifdef FORTE_USE_64BIT_DATATYPES
    setTUINT64(0);
  #else
    setTUINT32(0);
  #endif
  };
  virtual ~CIEC_ANY() {};

 /*! \brief Set method for data type member value
  *
  *  The data type value is set through the copy assignment
  *
  */
  void setValue(const CIEC_ANY& pa_poValue) {
    if (m_eDataTypeId <= e_TIME) { //quick assignment for primitive datatypes
      m_uAnyData = pa_poValue.m_uAnyData;
    }
    else {  //assignment for complex datatypes
      setValueComplex(&pa_poValue);
    }
  }

/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   Pure virtual function implementation.
 */
  virtual CIEC_ANY* clone(void) = 0;
/*! \brief Get data type id method
 *
 *   With this command the data type of the actual object can be identified.
 *   The data type identifier is unique for each data type within FORTE.
 */
  EDataTypeID getDataTypeID() const {
	  return m_eDataTypeId;
  }

/*! \brief Get the pointer to the union char array
 *
 *  This command returns the pointer to the union char array
 *  \return Returns TBYTE*
 */

  inline TBYTE* getDataPtr() {
    return m_uAnyData.m_AData;
  }

/*! \brief Converts string value to data type value
 *
 *   This command implements a conversion function from IEC61131
 *   data type (string format) to a C++ conform type.
 *   This function is necessary for communication with a proper engineering system.
 *   Pure virtual function implementation.
 */
  virtual bool fromString(const string &pa_rsValue) = 0;
/*! \brief Converts data type value to string
 *
 *   This command implements a conversion function from C++ data type
 *   to IEC61131 conform data type (string format).
 *   This function is necessary for communication with a proper engineering system.
 *   Pure virtual function implementation.
 */
  virtual bool toString(string &pa_rsValue) = 0;

/*! \brief determine whether we can cast the source to the destination and what kind of cast it is
 */
	static void isCastable(EDataTypeID pa_eSource, EDataTypeID pa_eDestination, bool &pa_rbUp, bool &pa_rbDown, bool &pa_rbCross);

  friend class CDataConnection;
  friend class CInternal2InterfaceDataConnection;
  friend class CIEC_ARRAY;

protected:
/*! \brief Set Method for complex datatypes
 *  A virtual function for datatypes who can't be copied by the union assignment
 */
virtual void setValueComplex(const CIEC_ANY*){};

/*! \brief Get Method for complex datatypes
 *  A virtual function for datatypes who can't be copied by the union assignment
 */

void setTBOOL8 (TBOOL8 src) {
  m_uAnyData.m_nUInt64 = TUINT64(src);
}
void setTUINT32 (TUINT32 src) { //also used for TDWORD
  m_uAnyData.m_nUInt64 = TUINT64(src);
}
void setTUINT16 (TUINT16 src) { //also used for TWORD
  m_uAnyData.m_nUInt64 = TUINT64(src);
}
void setTUINT8 (TUINT8 src) { //also used for TBYTE
  m_uAnyData.m_nUInt64 = TUINT64(src);
}
void setTINT32 (TINT32 src) {
  m_uAnyData.m_nInt64 = TINT64(src);
}
void setTINT16 (TINT16 src) {
  m_uAnyData.m_nInt64 = TINT64(src);
}
void setTINT8 (TINT8 src) {
  m_uAnyData.m_nInt64 = TINT64(src);
}

#ifdef FORTE_USE_FLOAT_DATATYPES
void setTFLOAT (TFLOAT src) {
  m_uAnyData.m_fFloat = TFLOAT(src);
}
#ifdef FORTE_USE_64BIT_DATATYPES
void setTDFLOAT (TDFLOAT src) {
  m_uAnyData.m_fDFloat = TDFLOAT(src);
}
#endif //#ifdef FORTE_USE_64BIT_DATATYPES
#endif //#ifdef FORTE_USE_FLOAT_DATATYPES

#ifdef FORTE_USE_64BIT_DATATYPES
void setTUINT64 (TUINT64 src) { //also used for LWORD
  m_uAnyData.m_nUInt64 = TUINT64(src);
}
void setTINT64 (TINT64 src) {
  m_uAnyData.m_nUInt64 = TUINT64(src);
}
#endif //#ifdef FORTE_USE_64BIT_DATATYPES

#ifdef FORTE_BIG_ENDIAN
TBOOL8 getTBOOL8() const {
  return (TBOOL8)m_uAnyData.m_nUInt64;
}
TUINT32 getTUINT32() const {  //also used for TDWORD
  return (TUINT32)m_uAnyData.m_nUInt64;
}
TUINT16 getTUINT16() const {  //also used for TWORD
  return (TUINT16)m_uAnyData.m_nUInt64;
}
TUINT8 getTUINT8() const {  //also used for TBYTE
  return (TUINT8)m_uAnyData.m_nUInt64;
}
TINT32 getTINT32() const {
  return (TINT32)m_uAnyData.m_nInt64;
}
TINT16 getTINT16() const {
  return (TINT16)m_uAnyData.m_nInt64;
}
TINT8 getTINT8() const {
  return (TINT8)m_uAnyData.m_nInt64;
}

#ifdef FORTE_USE_64BIT_DATATYPES
TUINT64 getTUINT64() const { //also used for LWORD
  return (TUINT64)m_uAnyData.m_nUInt64;
}
TINT64 setTINT64() const {
  return (TINT64)m_uAnyData.m_nUInt64;
}
#endif //#ifdef FORTE_USE_64BIT_DATATYPES
#else
#ifdef FORTE_LITTLE_ENDIAN
TBOOL8 getTBOOL8 () const {
  return m_uAnyData.m_bBool;
}
TUINT32 getTUINT32 () const { //also used for TDWORD
  return m_uAnyData.m_nUInt32;
}
TUINT16 getTUINT16 () const { //also used for TWORD
  return m_uAnyData.m_nUInt16;
}
TUINT8 getTUINT8 () const { //also used for TBYTE
  return m_uAnyData.m_nUInt8;
}
TINT32 getTINT32 () const {
  return m_uAnyData.m_nInt32;
}
TINT16 getTINT16 () const {
  return m_uAnyData.m_nInt16;
}
TINT8 getTINT8 () const {
  return m_uAnyData.m_nInt8;
}

#ifdef FORTE_USE_64BIT_DATATYPES
TUINT64 getTUINT64() const { //also used for LWORD
  return m_uAnyData.m_nUInt64;
}
TINT64 getTINT64() const {
  return m_uAnyData.m_nInt64;
}
#endif //#ifdef FORTE_USE_64BIT_DATATYPES

#else
#error Endianess not defined!
#endif //#ifdef FORTE_BIG_ENDIAN
#endif //#ifdef FORTE_LITTLE_ENDIAN

#ifdef FORTE_USE_FLOAT_DATATYPES //!< get-Methods are Big/Little Endian independent
TFLOAT getTFLOAT() const {
  return (TFLOAT)m_uAnyData.m_fFloat;
}
#ifdef FORTE_USE_64BIT_DATATYPES
TDFLOAT getTDFLOAT() const {
  return TDFLOAT(m_uAnyData.m_fDFloat);
}
#endif //#ifdef FORTE_USE_64BIT_DATATYPES
#endif //#ifdef FORTE_USE_FLOAT_DATATYPES

private:

  union UAnyData {
      TBOOL8 m_bBool;

      TBYTE m_cByte;
      TWORD m_cWord;
      TDWORD m_cDWord;

      TINT8 m_nInt8;
      TINT16 m_nInt16;
      TINT32 m_nInt32;

      TUINT8 m_nUInt8;
      TUINT16 m_nUInt16;
      TUINT32 m_nUInt32;

#ifdef FORTE_USE_FLOAT_DATATYPES
      TFLOAT m_fFloat;
#ifdef FORTE_USE_64BIT_DATATYPES
      TDFLOAT m_fDFloat;
#endif //#ifdef FORTE_USE_64BIT_DATATYPES
#endif //#ifdef FORTE_USE_FLOAT_DATATYPES

#ifdef FORTE_USE_64BIT_DATATYPES
      TINT64 m_nInt64;
      TUINT64 m_nUInt64;
      TBYTE m_AData[sizeof(TUINT64)]; //!< For data extraction in big endian machines
#else
      TBYTE m_AData[sizeof(TUINT32)]; //!< For data extraction in big endian machines
#endif //#ifdef FORTE_USE_64BIT_DATATYPES
  };

	EDataTypeID m_eDataTypeId;
	UAnyData m_uAnyData;
};

/*!\brief Type for handling CIEC_ANY pointers
 */
typedef CIEC_ANY* TIEC_ANYPtr;

typedef CIEC_ANY IIEC_ANY; //TODO: for legacy support, will be deleted with next major release

#endif /*_MANY_H_*/
