/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _FORTE_ARRAY_H_
#define _FORTE_ARRAY_H_

#include "any_der.h"

/*!\ingroup COREDTS CIEC_ARRAY represents the array data type according to IEC 61131.
 *  @author sch
 *  @version 20081121/mmm - changed constructor, renamed setValue to setValueComplex, removed getValue
 *  @version 20070129/sch - Initial version
 */
class CIEC_ARRAY : public CIEC_ANY_DERIVED {
  //DECLARE_FIRMWARE_DATATYPE(ARRAY)
private:
/*! \brief CIEC_ARRAY data type member value is a list of pointers to CIEC_ANY.
 *
 *   This list contains the elements in the array of the type given by
 *   the reference pointer.
 *
 */
  TIEC_ANYPtr *m_apoValue;

/*! \brief CIEC_ARRAY data type reference value is a pointer to IIEC_ANY.
 *
 *   This reference value will be used to instantiate new elements
 *   for the array.
 */
  CIEC_ANY *m_poReferenceValue;

/*! \brief CIEC_ARRAY array length.
 */
  TUINT16 m_nLength;

protected:
/*! \brief Set method for data type member value
 *
 *   With this command the connector data type of connected FBs can be set
 *   according to given connectors.
 *   \param pa_poValue  Pointer to given connector.
 *   \return Can be the following response:
 *     - No return value.
 */
  virtual void setValueComplex(const CIEC_ANY* pa_poValue);

public:
  //CIEC_ARRAY() : CIEC_ANY_DERIVED(e_ARRAY) { m_apoValue = 0; m_nLength = 0; m_poReferenceValue = 0; }

  CIEC_ARRAY(CIEC_ANY& pa_poRefVal);


  CIEC_ARRAY(TUINT16 pa_nLength, CIEC_ANY *pa_poRefVal);


  explicit CIEC_ARRAY(const CIEC_ARRAY& pa_roValue);


  ~CIEC_ARRAY();

/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - A clone of the actual data type object.
 */
  virtual CIEC_ANY* clone(void) { return new CIEC_ARRAY(*this); }

/*! \brief Get reference pointer of the elements
 *
 *   With this command the reference pointer of the elements can be retrieved.

 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - reference pointer.
 *     - NULL if not reference pointer is available
 */
  CIEC_ANY* getReferencePointer(){
    return m_poReferenceValue;
  };
/*! \brief Get the array's size
 *
 *   With this command the size of the array can be evaluated.
 *
 *   \param - No parameters necessary.
 *   \return - the length of the array.
 */
  virtual TUINT16 size() const{
    return m_nLength;
  };
/*! \brief Resizes the array
 *
 *   With this command the array size can be changed.
 *   After this operation any previous data is lost.
 *
 *   \param size new array size.
 *   \return - No return value.
 */
  void resize(TUINT16 size, CIEC_ANY *pa_poRefVal);
/*! \brief Resizes the array and change array element types
 *
 *   With this command the array size can be changed
 *   and also the type of the elements.
 *   After this operation any previous data is lost.
 *
 *   \param size new array size.
 *   \param pa_poRefVal reference pointer to element type
 *   \return - No return value.
 */
  void resize(TUINT16 size);
/*! \brief Resizes the array
 *
 *   Delete all element entries.
 *   After this operation any previous data is lost.
 *
 *   \param - No parameters necessary.
 *   \return - No return value.
 */
  void clear();

/*! \brief Operator: CIEC_INT data type [unsigned int] data type
 *
 *   This command implements the index operator for a C++ unsigned int index.
 *   \param pa_nIndex  Index
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_ANY* operator [] (TUINT16 pa_nIndex){
    if ( m_nLength > pa_nIndex ) {
      return m_apoValue[pa_nIndex];
    }
    return 0;
  };

  const CIEC_ANY* operator [] (TUINT16 pa_nIndex) const{
    if ( m_nLength > pa_nIndex ) {
      return m_apoValue[pa_nIndex];
    }
    return 0;
  };

/*! \brief Converts array value to data type value
 *
 *   This command implements a conversion function from IEC61131
 *   data type (array format) to a C++ conform type.
 *   This function is necessary for communication with a proper engineering system.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successful - something went wrong!
 *     -  true....conversion was successful.
 */
  virtual bool fromString(const string& pa_rsValue);
/*! \brief Converts data type value to array
 *
 *   This command implements a conversion function from C++ data type
 *   to IEC61131 conform data type (array format).
 *   This function is necessary for communication with a proper engineering system.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successful - something went wrong!
 *     -  true....conversion was successful.
 */
  virtual bool toString(string& pa_rsValue);
};

#endif /*_FORTE_ARRAY_H_*/
