/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _FORTE_LREAL_H_
#define _FORTE_LREAL_H_

#include "any_rea.h"

/*!\ingroup COREDTS CIEC_LREAL represents the lreal data type according to IEC 61131.
 *  @author tstras & imuell
 *  @version 20081112/mmm - changed deviation from IIEC_ANY_REAL to CIEC_ANY_REAL due to changing class structure
 *  						changed order of declarations to public - protected - due to design rules changes
 *  						corrected spelling in comments; deleted m_fValue, getDataTypeID, getValue and setValue due to changing class structure
 *  @version 20070307/az - changed serialization code for supporting also the
 *                         Serialization of arrays.
 *  @version 20061108/az - added in sertion ito data type library in the typelib
 *  @version 20061107/gebenh - getDataPtr() method added, serializedouble() and
 *                             deserializedouble() moved from IIEC_ANY to here
 *  @version 20060719/tstras - deletion of mathematical operators,
 *                             conversion operator added
 *  @version 20060707/tstras - bugfix in fromString() method
 *  @version 20060322/tstras - isFacadeDataType() method added, bugfix for
 *                             handling FACADE data type in setValue() &
 *                             getValue() methods
 *  @version 20060321/tstras - bug for FACADE data type in setValue() &
 *                             getValue() methods fixed
 *  @version 20051205/tstras - clone() method added
 *  @version 20051201/tstras - getDataTypeID() method, setValue(IIEC_ANY*) &
 *                             getValue(IIEC_ANY*) added
 *  @version 20051027/tstras & imuell - Initial version
 */
class CIEC_LREAL : public CIEC_ANY_REAL {
  DECLARE_FIRMWARE_DATATYPE(LREAL)

  public:
  CIEC_LREAL() : CIEC_ANY_REAL(e_LREAL) { }
  explicit CIEC_LREAL(const CIEC_LREAL& pa_roValue): CIEC_ANY_REAL(e_LREAL) {
    setValue(pa_roValue);
  }
  explicit CIEC_LREAL(TDFLOAT pa_fValue) : CIEC_ANY_REAL(e_LREAL) {
    setTDFLOAT(pa_fValue);
  }
  virtual ~CIEC_LREAL() {}
/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - A clone of the actual data type object.
 */
  virtual CIEC_ANY* clone(void) { return new CIEC_LREAL(*this); }

/*! \brief Operator: CIEC_LREAL data type = CIEC_LREAL data type
 *
 *   This command implements the assignment operator for the C++ datatype DFLOAT.
 *   The parameter value is represented by a C++ variable (call by value).
 *   \param pa_fValue  Value for assignment.
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_LREAL& operator = (TDFLOAT pa_fValue) {
    setTDFLOAT(pa_fValue);
    return *this;
  }
/*! \brief Converts CIEC_LREAL to elementary 32 bit float
 *
 *   Conversion operator for converting CIEC_LREAL to elementary 32 bit float
 */

  operator TDFLOAT() const {
    return getTDFLOAT();
  }

/*! \brief Converts string value to data type value
 *
 *   This command implements a conversion function from IEC61131
 *   data type (string format) to a C++ conform type.
 *   This function is necessary for communication with a proper engineering system.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successful - something went wrong!
 *     -  true....conversion was successful.
 */
  virtual bool fromString(const string& pa_rsValue);
/*! \brief Converts data type value to string
 *
 *   This command implements a conversion function from C++ data type
 *   to IEC61131 conform data type (string format).
 *   This function is necessary for communication with a proper engineering system.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successfull - something went wrong!
 *     -  true....conversion was successfull.
 */
  virtual bool toString(string& pa_rsValue);

private:
/*! \brief bit distortion extension for serialization function - 64 bit values.
 *   \param pa_pcBytes  Pointer to first part of original object.
 *   \param pa_nValue   Pointer to the modified object
 *   \return Can be the following response:
 *     - No return value.
 */
  static void serializedouble(TBYTE* pa_pcBytes, unsigned long long pa_nValue){
    pa_pcBytes[0] = (TBYTE)((pa_nValue >> 56) & 0x00FF);
    pa_pcBytes[1] = (TBYTE)((pa_nValue >> 48) & 0x00FF);
    pa_pcBytes[2] = (TBYTE)((pa_nValue >> 40) & 0x00FF);
    pa_pcBytes[3] = (TBYTE)((pa_nValue >> 32) & 0x00FF);
    pa_pcBytes[4] = (TBYTE)((pa_nValue >> 24) & 0x00FF);
    pa_pcBytes[5] = (TBYTE)((pa_nValue >> 16) & 0x00FF);
    pa_pcBytes[6] = (TBYTE)((pa_nValue >> 8) & 0x00FF);
    pa_pcBytes[7] = (TBYTE)(pa_nValue & 0x00FF);
  };
/*! \brief bit distortion extension for deserialization function - 64 bit values.
 *   \param pa_pcBytes  Pointer to first part of final object.
 *   \param pa_nValue   Pointer to the modified object
 *   \return Can be the following response:
 *     - No return value.
 */
  static void deserializedouble(const TBYTE* pa_pcBytes, unsigned long long* pa_nValue){
    *pa_nValue = ((((unsigned long long)pa_pcBytes[0]) << 56) & 0xFF00000000000000ULL) +
                 ((((unsigned long long)pa_pcBytes[1]) << 48) & 0x00FF000000000000ULL) +
                 ((((unsigned long long)pa_pcBytes[2]) << 40) & 0x0000FF0000000000ULL) +
                 ((((unsigned long long)pa_pcBytes[3]) << 32) & 0x000000FF00000000ULL) +
                 ((((unsigned long long)pa_pcBytes[4]) << 24) & 0x00000000FF000000ULL) +
                 ((((unsigned long long)pa_pcBytes[5]) << 16) & 0x0000000000FF0000ULL) +
                 ((((unsigned long long)pa_pcBytes[6]) << 8) & 0x000000000000FF00ULL) +
                 ((unsigned long long)pa_pcBytes[7] & 0x00000000000000FFULL);
  };

};

#endif /*_FORTE_LREAL_H_*/
