/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _FORTE_WSTRING_H_
#define _FORTE_WSTRING_H_

#include "any_str.h"

/*!\ingroup COREDTS \brief CIEC_WSTRING represents the wide string data type according to IEC 61131.
 *
 *  "wstring" is implemented like the "string" - the following first implementation
 *  "typedef std::basic_string<wchar_t> TWSTRING; didn't work well"
 *  @author tstras & imuell
 *  @version 20081113/mmm - deleted getDataPtr(), getDataTypeID(); changed IIEC_ANY_STRING to CIEC_ANY_STRING due to class structure change
 *  @version 20081121/mmm - removed setValue, getValue, added setValueComplex, removed getDataTypeID
 *    changed constructors, renamed class due to class structure change
 *  @version 20070307/az - changed serialisation code for supporting also the
 *                         serialisation of arrays.
 *  @version 20061108/az - added in sertion ito data type library in the typelib
 *  @version 20061107/gebenh - getDataPtr() method added
 *  @version 20060717/az & tstras - bugfix serialize() method
 *  @version 20060719/tstras - deletion of mathematical operators,
 *                             conversion operator added
 *  @version 20060322/tstras - isFacadeDataType() method added, bugfix for
 *                             handling FACADE data type in setValue() &
 *                             getValue() methods
 *  @version 20060321/tstras - bug for FACADE data type in setValue() &
 *                             getValue() methods fixed
 *  @version 20060214/imuell - wstring impementation similar to string implementation
 *  @version 20051205/tstras - clone() method added
 *  @version 20051201/tstras - getDataTypeID() method, setValue(IIEC_ANY*) &
 *                             getValue(IIEC_ANY*) added
 *  @version 20051121/tstras & imuell - Initial version
 */
class CIEC_WSTRING : public CIEC_ANY_STRING {
  DECLARE_FIRMWARE_DATATYPE(WSTRING)
public:
  CIEC_WSTRING() : CIEC_ANY_STRING(e_WSTRING) {
  }
  explicit CIEC_WSTRING(const CIEC_WSTRING& pa_roValue) : CIEC_ANY_STRING(e_WSTRING) {
    m_sValue = pa_roValue.m_sValue;
  }
  explicit CIEC_WSTRING(string pa_sValue) : CIEC_ANY_STRING(e_WSTRING) {
    m_sValue = pa_sValue;
  }
  virtual ~CIEC_WSTRING() {}
/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - A clone of the actual data type object.
 */
  virtual CIEC_ANY* clone(void) { return new CIEC_WSTRING(*this); }

/*! \brief Set-Method for CIEC_WSTRING variable
 *
 *   With this command the value of the actual object can be set.
 *   \param pa_sValue  Object will be set to this value.
 *   \return Can be the following response:
 *     - No return value.
 */
  void setValue(string pa_sValue) { m_sValue = pa_sValue; }
/*! \brief Get-Method for CIEC_WSTRING variable
 *
 *   With this command the value of the actual object can be read.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - Actual value of the object.
 */
  string getValue(void) { return m_sValue; }

/*! \brief Operator: CIEC_WSTRING data type = wchar_t data type
 *
 *   This command implements the assignment operator for the C++ datatype WSTRING
 *   The parameter value is represented by a C++ variable (call by value).
 *   \param pa_sValue  Value for assignment.
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_WSTRING& operator= (const string &pa_sValue) {
    m_sValue = pa_sValue;
    return *this;
  }
/*! \brief Converts CIEC_WSTRING to elementary string
 *
 *   Conversion operator for converting CIEC_WSTRING to elementary string
 */
  operator const string &() const { return m_sValue; };
/*! \brief Converts CIEC_STRING to elementary string
 *
 *   Conversion operator for converting CIEC_STRING to elementary string
 */
  operator const char* () const { return m_sValue.c_str(); };


/*! \brief Converts string value to data type value
 *
 *   This command implements a conversion function from IEC61131
 *   data type (string format) to a C++ conform type.
 *   This function is necessary for communication with a proper engineering system.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successfull - something went wrong!
 *     -  true....conversion was successfull.
 */
  virtual bool fromString(const string& pa_rsValue){
    m_sValue = pa_rsValue;
    return true;
  };
/*! \brief Converts data type value to string
 *
 *   This command implements a conversion function from C++ data type
 *   to IEC61131 conform data type (string format).
 *   This function is necessary for communication with a proper engineering system.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successfull - something went wrong!
 *     -  true....conversion was successfull.
 */
  virtual bool toString(string& pa_rsValue){
    pa_rsValue = m_sValue;
    return true;
  };

protected:
/*! \brief Set method for data type member value
 *
 *   With this command the connector data type of connected FBs can be set
 *   according to given connectors.
 *   \param pa_poValue  Pointer to given connector.
 *   \return Can be the following response:
 *     - No return value.
 */
  virtual void setValueComplex(const CIEC_ANY* pa_poValue){
    m_sValue = ((CIEC_WSTRING*)pa_poValue)->m_sValue;
  };

private:
  string m_sValue;
};

#endif /*_FORTE_WSTRING_H_*/
