/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _DEVICE_H
#define _DEVICE_H

#include <list>
#include "objhand.h"
#include "typelib.h"
#include "devexec.h"

/*!\ingroup CORE CDevice represents a device according to IEC 61499. CDevice contains
      - one or more IEC 61499 compliant resources (CResource),
      -	a device management (CDeviceAdministrator)
      -	a function block execution environment (CFBNExecution)
      -	a library (CTypeLib) which administrates the types the 1499 device is
        capable of
      - External event interface for handling interrupts and other external notifications.
      -	FORTE-HAL: FORTE Hardware Abstraction Layer

    There are different device types available (e.g. remote device RMT_DEV)
    for different tasks which have different device configurations
    (resources function blocks ) and device parameters.
 * @author AZ
 * @version 20070911/AZ - renamed execute() to startDevice()
 * @version 20060719/AZ - changed base class to C61499ObjectHandler in order to simplify management commands
 * @version 20050701/AZ - Initial version as basis for the interface documentation
 *
*/
class CDevice : public C61499ObjectHandler{
  private:
/*! \brief
 *
 */
    CDeviceExecution m_oDeviceExecution;

  protected:
    virtual void executeEvent(TEventID ){};

    virtual CFunctionBlock *createFBObject(CStringDictionary::TStringId pa_nInstanceNameId, CStringDictionary::TStringId pa_nResTypeId, CStringDictionary::TStringId pa_nApplId);
    virtual bool deleteFBObject(CFunctionBlock *pa_poFBObj){
      return m_roTypeLib.deleteResource((CResource *)pa_poFBObj);
    };
  public:
/*! \brief Sets up all the necessary data and classes necessary for execution.
 *
 */
    CDevice(const SFBInterfaceSpec *pa_pstInterfaceSpec,
            TDataConnectionPtr *pa_apoDIConns, TIEC_ANYPtr *pa_apoDIs):
         C61499ObjectHandler(0, 0, pa_pstInterfaceSpec, 0, pa_apoDIConns, pa_apoDIs, 0, 0),
         m_oDeviceExecution(){
    };
    virtual ~CDevice(){};

    virtual CStringDictionary::TStringId getFBTypeId(void) const { return CStringDictionary::scm_nInvalidStringId; };

/*! \brief Starts to execute FBNs.
 *
 *  This function will send a IEC 61499 start command to all existing resources in the device (if any).
 *  The function will return after the start. The Forte HAL has to ensure that it is waited till the device
 *  finishes its execution.
 *  \return 0 on success -1 on error
 */
    virtual int startDevice(void){
      startManagedObject();
      return 1;
    };
/*!\brief Execute the given mangement command
 *
 * Evaluates the m_sDestination parameter of the command if empty this class tries to execute the management command if not
 * the referenced resource is called for handling the command.
 * \param pa_oCommand FORTE internal representation of the management command
 * \return response of the MGMCommand execution as defined in IEC 61499
 */
   virtual EMGMResponse executeMGMCommand(SManagementCMD &pa_oCommand);

//! Retrieve the device execution of this device
    CDeviceExecution &getDeviceExecution(void) {return m_oDeviceExecution; };
};

#endif
