/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/ 
#ifndef _EVENCONN_H_
#define _EVENCONN_H_

#include "conn.h"
#include "event.h"

class CEventChainExecutionThread;

/*!\ingroup CORE \brief Class for handling an event connection.
 *  
 * 
 * @author AZ
 * @version 20060721/AZ - changed storage for the destination fbs and in events. will be the basis for
 *                        improved event chain executors
 * @version 20060214/AZ - Fixed bug in triggerEvent() there where to for loops instead of one
 * @version 20060206/AZ - Changed event connection handling
 * @version 20050920/AZ - Initial version as basis for the interface documentation
 */
class CEventConnection : public CConnection {
private:
/*!\brief Structure to hold the information needed for storing event entry list in the event
 * connection 
 */
  struct SECEventListEntry{
    SEventEntry m_stEventEntry; //! the event entry to be stored.
    SECEventListEntry *m_pstNext; //! pointer to the next element in the list. if 0 this is the last entry in the list
  };
  SECEventListEntry *m_pstDestinationsStart; //! Pointer to the start of the connected events list
protected:
public:
  CEventConnection(TConnectionID pa_nSourceId) : CConnection(pa_nSourceId){
    m_pstDestinationsStart = 0;                  
  };
  CEventConnection(const char *pa_acFBName, const char *pa_acOutputName) : CConnection(CConnection::genConPortId(pa_acFBName, pa_acOutputName)){
    m_pstDestinationsStart = 0;
  };
  virtual ~CEventConnection();
  /*! \brief Establishes the connection.
 * 
 * With this command the connection is created and the FBs are set to apropriate states.
 * This mainly efects the destionation of the connection (because of the fan-out)
 * If the referenced connection through the pa_sDst parameter is the first destionation
 * also the src reference is created as necessary.
 * \param pa_nDstId     string of the destionation of the connection
 * \param pa_roSrcFB  reference to the source FB of this connection
 * \param pa_roDstFB  reference to the destionation Fb of this connection
 * \param pa_sDstInput name of the input of the destination FB
 * \return can be the following response:
 *     - e_RDY... creation went well.  
 *     - e_NO_SUCH_OBJECT... The destionation is not a valid input.
 *     - e_INVALID_STATE... The specified connection already exists.
 */  
  virtual EMGMResponse connect(TConnectionID pa_nDstId, CFunctionBlock &pa_roSrcFB, CFunctionBlock &pa_roDstFB, const char *pa_sDstInput);
  
  virtual EMGMResponse connect(const char *pa_acDestFBID, CFunctionBlock &pa_roSrcFB, CFunctionBlock &pa_roDstFB, const char *pa_sDstInput){
    return connect(CConnection::genConPortId(pa_acDestFBID, pa_sDstInput),
                  pa_roSrcFB, pa_roDstFB, pa_sDstInput);
  };
/*! \brief Disconnects the connection.
 *
 * With this command the connection is removed and the FBs are set to apropriate states.
 * This mainly efects the destionation of the connection (because of the fan-out)
 * If the referenced connection through the pa_sDst parameter is the last destionation
 * also the src reference is removed.
 * \param pa_nDstId     string of the destionation of the connection
 * \param pa_roSrcFB  reference to the source FB of this connection
 * \param pa_roDstFB  reference to the destionation Fb of this connection
 * \param pa_sDstInput name of the input of the destination FB
 * \return can be the following response:
 *     - e_RDY... creation went well.
 *     - e_NO_SUCH_OBJECT... The destionation is not a valid input.
 */
  virtual EMGMResponse disconnect(TConnectionID pa_nDstId, CFunctionBlock &pa_roSrcFB, CFunctionBlock &pa_roDstFB, const char *pa_sDstInput);
/*! \brief Triggers the event connection and all destinations are notified.
 *
 *
 *  \param pa_poExecEnv Pointer to the execution environment the event is sent in.
 */
  virtual void triggerEvent(CEventChainExecutionThread &pa_poExecEnv);
};

typedef CEventConnection *TEventConnectionPtr;

#endif /*_EVENCONN_H_*/
