/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 * 			2008-04-24: ih, hegny@acin.tuwien.ac.at - added adapter support
 *******************************************************************************/
#ifndef _FUNCBLOC_H_
#define _FUNCBLOC_H_

#include "manobj.h"
#include "event.h"
#include "typelib.h"
#include "dataconn.h"
#include "eventconn.h"
#include "adapterconn.h"
#include "stringdict.h"
#include "../arch/devlog.h"

class CEventChainExecutionThread;
class CResource;
class CAdapter;

typedef char *TFBNameListEntry; //!< \ingroup CORE Type for storing an entry of the different namelists of an FB. e.g. EO names.
typedef TUINT8 TDataIOID; //!< \ingroup CORE Type for holding an data In- or output ID (max value 254)

//! \ingroup CORE Constant for the invalid data output id
static const TDataIOID cg_nInvalidDataOutputID = 255;

//! \ingroup CORE Constant for the invalid data input id
static const TDataIOID cg_nInvalidDataInputID = 255;

/*!\ingroup CORE\brief Structure to hold all the data for specifying a function block interface.
 */
struct SFBInterfaceSpec{
  TUINT8 m_nNumEIs;                     //!< Number of event inputs (max 254)
  const TFBNameListEntry *m_acEINames;  //!< List of the event input names
  const TDataIOID *m_anEIWith;          //!< Input WITH reference list. This list contains an array of input data ids. For each input event the assiciated data inputs are listed. The start for each input event is specified in the m_anEIWithIndexes field. The end is defined trough the value 255.
  const TINT16 *m_anEIWithIndexes;      //!< Index list for each input event. This list gives for each input event an entry in the m_anEIWith. Input events are numbered starting from 0. if the input event has no assciated data inputs -1 is the entry at this event inputs postion.
  TUINT8 m_nNumEOs;                     //!< Number of event outputs (max 254)
  const TFBNameListEntry *m_acEONames;  //!< List of the event output names
  const TDataIOID *m_anEOWith;          //!< Output WITH reference list. This list contains an array of output data ids. For each output event the assiciated data outputs are listed. The start for each output event is specified in the m_anEOWithIndexes field. The end is defined trough the value 255.
  const TINT16 *m_anEOWithIndexes;      //!< Index list for each output event. This list gives for each output event an entry in the m_anEOWith. Output events are numbered starting from 0. if the output event has no assciated data outputs -1 is the entry at this event outputs postion. Additionally at the postion m_nNumEOs in this list an index to an own list in the m_anEOWith list is stored specifying all output data port that are not associated with any output event. That values will be updated on every FB invocation.
  TUINT8 m_nNumDIs;                     //!< Number of data inputs (max 254)
  const TFBNameListEntry *m_acDINames;  //!< List of the data input names
  TUINT8 m_nNumDOs;                     //!< Number of data outputs (max 254)
  const TFBNameListEntry *m_acDONames;  //!< List of the data output names
};

/*!\ingroup CORE\brief Base class for all function blocks.
 *
 * Open TODOs come up by introduction of the new base class CManagedObject
 *    - startManagedObject: TODO: add function to runECC
 *    - stopManagedObject: TODO: add function complete algorithm
 *        Comment AZ: I don't think this should be done here because the algorithm is part of some event chain.
 *        Therefore it should be complited there but the state change should result in no furhter evalutaiton of the ECC.
 *    - killManagedObject: TODO: add function to kill algorithm execution
 *    - resetManagedObject: TODO: add function to initialize FB. Maybe the same as the things that have to be done in the constructor (-> initFB() ).
 *
 * @author GuGr
 * @version 20081121/mmm - added getDIID, getDOID; changed connectDI, connectDO, getDataInput, getDataOutput
 * @version 20080108/IH - Added support for adapters:
 * @version 20070329/HT - Added 2 methods (getInternalVar, getInternalNames) and 2 variables in SFBInterfaceSpec
 * @version 20060921/AZ,SMR - Removed string parameter in ConfigureFB()
 * @version 20060619/AZ,SMR - New functionblock interface introduced
 * @version 20060208/AZ - Fixed sam smoller bugs in the for statements with the new event connection mechanism
 * @version 20060207/AZ - Removed number of EOs in CFunctionBlock-Constructor, some smaller bug fixes for error checking when
 *                        the fb has no event in- or outputs.
 * @version 20060206/AZ - Changed event connection handling
 * @version 20051206/tstras - connectDI(), connectDO(), writeDataOutput() &
 *                            readDataInput() methods added
 * @version 20051108/AZ - moved the running states to the base class CManagedObject
 * @version 20051021/AZ - added support for start, stop, kill and the fb state
 * @version 20051019/GuGr - First version as basis for interface description
 */
class CFunctionBlock : public CManagedObject{
private:
  CStringDictionary::TStringId m_nApplId;		   //!< The applicationId the function block is a member of.
	CResource *m_poResource;                     //!< A pointer to the resource containing the function block.
protected:
  const SFBInterfaceSpec *cm_pstInterfaceSpec; //!< Pointer to the interface specification
	CStringDictionary::TStringId m_nFBInstanceName; //!< the instance name of the object
  TEventConnectionPtr *m_apoEOConns;           //!< A list of event connections pointers storing for each event output the event connection. If the output event is not connected the pointer is 0.
  TDataConnectionPtr *m_apoDIConns;            //!< A list of data connections pointers storing for each data input the data connection. If the data input is not connected the pointer is 0.
  TIEC_ANYPtr *m_apoDIs;                       //!< A list of pointers to the data inputs. This allows to implement a general getDataInput()
  TDataConnectionPtr *m_apoDOConns;            //!< A list of data connections pointers storing for each data output the data connection. If the data output is not connected the pointer is 0.
  TIEC_ANYPtr *m_apoDOs;                       //!< A list of pointers to the data outputs. This allows to implement a general getDataOutput()
  CEventChainExecutionThread *m_poInvokingExecEnv;      //!< A pointer to the execution thread that invoked the FB. This value is taored here to reduce function paramters and reduce therefore stack usage.
  CAdapter **m_apoAdapters;                    //!< A list of pointers to the adapters. This allows to implement a general getAdapter().
  TUINT8 m_nNumAdapters;                       //!< Number of the Adapters (plugs and sockets)

/*!\brief Function providing the functionality of the FB (e.g. execute ECC for basic FBs).
 *
 * \param pa_nEIID Event input ID where event occured.
 * \param pa_poExecEnv Eventchain within the event is executed.
 */
  virtual void executeEvent(TEventID pa_nEIID) = 0;
/*!\brief Function to send an output event of the FB.
 *
 * \param pa_nEO Event output ID where event should be fired.
 * \param pa_poExecEnv Eventchain within the event is executed.
 */
  void sendOutputEvent(TEventID pa_nEO);
public:
/*!\brief The main constructur for a function block.
 *
 * \param pa_acAppl  name of the application this function block belongs to
 * \param pa_poSrcRes pointer to the resource this function block is contained in (mainly necessary for management functions and service interfaces)
 * \param pa_pstInterfaceSpec const pointer to the interface spec
 * \param pa_apoEOConns an array of event connections pointers needs to be already allocated. The FB contstructor will initialize it with 0s.
 * \param pa_apoDIConns an array of data connections pointers needs to be already allocated. The FB constructor will initialize it witz 0s.
 * \param pa_apoDIs an array of CIEC_ANY pointers, holding pointers to all data inputs.
 * \param pa_apoDOConns an array of data connections pointers needs to be already allocated. The FB constructor will initialize it with 0s.
 * \param pa_apoDOs an array of CIEC_ANY pointers, holding pointers to all data outputs.
 */
    CFunctionBlock(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes,
                 const SFBInterfaceSpec *pa_pstInterfaceSpec,
                 TEventConnectionPtr *pa_apoEOConns,
                 TDataConnectionPtr *pa_apoDIConns,
                 TIEC_ANYPtr *pa_apoDIs,
                 TDataConnectionPtr *pa_apoDOConns,
                 TIEC_ANYPtr *pa_apoDOs,
                 TUINT8 pa_nNumAdapters = 0, CAdapter **pa_apoAdapters = 0);
	virtual ~CFunctionBlock() {};
/*!\brief Get the string of the application the function block is a member of.
 */
 	CStringDictionary::TStringId getApplId(void) const {
    return m_nApplId;
  };
 /*!\brief Get the resource the function block is contained in.
  */
  CResource &getResource(void){return *m_poResource;};
/*!\brief Returns the type of this FB instance
 */
  virtual CStringDictionary::TStringId getFBTypeId(void) const = 0;
 /*!\brief Get the ID of a specific event input of the FB.
 *
 * \param pa_acEIName Pointer to the event input name.
 * \return The ID of the event input or cg_nInvalidEventID.
 */
  TEventID getEIID(const char *pa_acEIName);
/*!\brief Check if the FB has an event output of a given name.
 *
 * \param pa_acEOName Pointer to the name of the event output to check.
 * \return TRUE if event output exists.
 */
  bool hasEventOutput(const char *pa_acEOName);
 /*!\brief Connects specific event output of a FB with a specific event connection.
 *
 * With this command a event output of the FB is connected to a specific connection.
 * \param pa_acEOName Pointer to the event output name.
 * \param pa_poEventCon Pointer to the event connection the FB should be connected to.
 * \return TRUE on success, FALSE if event output not exists or output is already connected.
 */
  bool connectEO(const char *pa_acEOName, CEventConnection *pa_poEventCon);

 /*!\ brief Gets the index of the m_acDINames array of a specific data output of a FB
  *
  * \param pa_acDIName Reference to the data input name.
  * \return Returns index of the Data Input Array of a FB
  */
  virtual unsigned int getDIID(const char *pa_acDIName) const;

 /*!\ brief Gets the index of the m_acDONames array of a specific data output of a FB
  * \param pa_acDOName Reference to the data input name.
  * \return Returns index of the Data Output Array of a FB
  */
  virtual unsigned int getDOID(const char *pa_acDOName) const;

/*!\brief Connects specific data input of a FB with a specific data connection.
 *
 * \param pa_nDIID ID of the data input name.
 * \param pa_poDataCon Pointer to the data connection the FB should be connected to.
 * \return TRUE on success, FALSE if data output not exists or output is already connected.
 */
  virtual bool connectDI(unsigned int pa_nDIID, CDataConnection *pa_poDataCon);
/*!\brief Connects specific data output of a FB with a specific data connection.
 *
 * \param pa_nDOID ID of the data output name.
 * \param pa_poDataCon Pointer to the data connection the FB should be connected to.
 * \return TRUE on success, FALSE if data output not exists or output is already connected.
 */
  virtual bool connectDO(unsigned int pa_nDOID, CDataConnection *pa_poDataCon);

/*!\brief Get the pointer to a data output of the FB.
 *
 * \param pa_acDOID ID of the data output name.
 * \return Pointer to the data output or 0. If 0 is returned DataOutput is ANY
 */
  virtual CIEC_ANY* getDataOutput(unsigned int pa_nDOID);
/*!\brief Get the pointer to a data input of the FB.
 *
 * \param pa_nDIID ID of the data input name.
 * \return Pointer to the data input or 0. If 0 is returned DataInput is ANY
 */
  virtual CIEC_ANY* getDataInput(unsigned int pa_nDIID);
/*!\brief Get the pointer to a variable of the FB.
 *
 * \param pa_acAdapterName Reference to the variable name.
 * \return Pointer to the variable or 0.
 *  The variable may be input, output or in the case of a BFB an internal var.
 */
  virtual CIEC_ANY* getVar(const char *pa_acAdapterName);
/*!\brief Get the pointer to the adapter instance of the FB.
 *
 * \param pa_acAdapterName Reference to the adapter name.
 * \return Pointer to the data output or 0.
 */
  virtual CAdapter* getAdapter(const char *pa_acAdapterName);

/*!\brief Function that handles incoming events.
 *
 * \param pa_nEIID ID if the input event that has occured.
 * \param pa_poExecEnv Event chain execution environment the FB will be executed in (used for adding output events).
 */
  void receiveInputEvent(TEventID pa_nEIID, CEventChainExecutionThread &pa_poExecEnv);
/*!\brief Configuration interface used by the typelib to parametrize generic function blocks.
 *
 * \param pa_acConfigString  A string containing the needed configuration data.
 * \return TRUE if configuration went ok, else FALSE.
 */
  virtual bool configureFB(const char *pa_acConfigString);

  virtual EMGMResponse startManagedObject(void);

  virtual EMGMResponse stopManagedObject(void);

  virtual EMGMResponse killManagedObject(void);

  virtual EMGMResponse resetManagedObject(void);


	friend class CInternal2InterfaceEventConnection;

/*!\brief Get/set the instance name
 */
	CStringDictionary::TStringId getInstanceNameId(void) const {
	  return m_nFBInstanceName;
	};

	const char *getInstanceName(void) const {
	  return CStringDictionary::getInstance().get(m_nFBInstanceName);
	}

	void setInstanceNameId(CStringDictionary::TStringId pa_nInstanceNameId) {
    m_nFBInstanceName = pa_nInstanceNameId;
  }
};

typedef CFunctionBlock *TFunctionBlockPtr;

#endif /*_FUNCBLOC_H_*/
