/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _MGMCMD_H_
#define _MGMCMD_H_

#include <string>
#include "stringdict.h"
#include "../arch/datatype.h"


typedef TUINT64 TConnectionID; //TODO think of making this a struct for 16Bit systems

/**  \ingroup CORE \defgroup MGMCommands Management Commands Internal Representation
 * \brief In this section the FORTE representation of the management commands is described.
 */
/*@{*/

/*!\brief Type for the management command (e.g. create, delete ...)
 */
typedef unsigned char TMGMCommandType;

/*!\brief All the data necessary for processing a managment command.
 *
 * The data of this structure is the internal FORTE representation of an IEC 61499 management command.
 * e.g. create <FB name="START" type="E_RESTART"> will result in the following data values:
 *    - m_nCMD = 0x30
 *    - m_sDestination = ""
 *    - m_sFistParam = "START" 
 *    - m_sSecondParam = "E_RESTART"
 *    - m_sAdditionalParams = ""
 */
struct SManagementCMD {
    /*!\brief An identification number for the requested command based on the definitons in the standard.
     * 
     * See the constants in devadmin.h for the available commands
     */
    TMGMCommandType m_nCMD;
    /*!\brief The destination of the command.
     * 
     * if the destination is scm_nInvalidStringID device is the target.
     * Otherwise a the TStringId of the resource's name the command 
     * should be exectued in is the target.
     * FIXME if more than the resource should be supported (e.g. composite FBs) the type needs to be changed .
     */
    CStringDictionary::TStringId m_nDestination;
    /*!\brief The first parameter of the command e.g. the FB name to create or the src of an connection.
     * The data is stored as
     *  - single TStringId for a FB name
     *  - as a combined TStringId for connections see CConnection for details 
     */
    TConnectionID m_nFirstParam;
    /*!\brief The second parameter of the command e.g. the FB type to create or the dst ("FBName.input") of an connection.
     * The data is stored as
     *  - single TStringId for a FB name
     *  - as a combined TStringId for connections see CConnection for details 
     */
    TConnectionID m_nSecondParam;
    /*!\brief Additional params needed by a MGM command (comment az: up to now i don't know of any such command. but maybe we will define anything. If not remove it for memory saving purposes)
     *        comment ge: used for the return value of query commands
     * 		  comment ht: used for the return value of the Application Name for FunctionBlocks
     */
    std::string m_sAdditionalParams;
};

//TODO insert all definitions for commands and descriptions important also the coresponding values in the MGM comand structure
//from the standard unterscheidung zwischen create/delete connection & FB
/**  \defgroup SuportedMGMCommands Supported Management Commands  
 * \ingroup MGMCommands
 * \brief In this section the definiton of the supported management commands are brought.
 * 
 * The constants for the management commands are seleceted in the folloing way:
 *  - the lower 4 bit are the same as the corresponding command defined by the standerd (see 61499-1: Table 6, p47).
 *  - the higher 4 bits are counted as they are listed in the Table 8, p48 in 61499-1.
 */
/*@{*/
/*! \brief Create a new data type defintion in the FORTE.
 */
const TMGMCommandType cg_nMGM_CMD_Create_DataType = 0x10;
/*! \brief Create a new FB type defintion in the FORTE.
 */
const TMGMCommandType cg_nMGM_CMD_Create_FBType = 0x20;
/*! \brief Create a new FB or resource instance.
 * 
 * When creating a FB instance the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for creating within the device or "resname" for creating within a resource
 *    - m_sFistParam = FB Instance name 
 *    - m_sSecondParam = FB Type
 *    - m_sAdditionalParams not used
 */
const TMGMCommandType cg_nMGM_CMD_Create_FBInstance = 0x30;
/*! \brief Create a new connection (can be event or data connection).
 * 
 * When creating a connection the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for creating within the device or "resname" for creating within a resource
 *    - m_sFistParam = source of the connection given by: "fbname.output"
 *    - m_sSecondParam = destination of the connection given by: "fbname.intput"
 *    - m_sAdditionalParams not used
 */
const TMGMCommandType cg_nMGM_CMD_Create_Connection = 0x40;
const TMGMCommandType cg_nMGM_CMD_Delete_DataType = 0x11;
const TMGMCommandType cg_nMGM_CMD_Delete_FBType = 0x21;
/*! \brief Delete a FB or resource instance.
 * 
 * When deleting a FB instance the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for deleteing within the device or "resname" for creating within a resource
 *    - m_sFistParam = FB Instance name 
 *    - m_sSecondParam not used
 *    - m_sAdditionalParams not used
 */
const TMGMCommandType cg_nMGM_CMD_Delete_FBInstance = 0x31;
/*! \brief Delete a connection (can be event or data connection).
 * 
 * When deleting a connection the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for deleting within the device or "resname" for deleting within a resource
 *    - m_sFistParam = source of the connection given by: "fbname.output"
 *    - m_sSecondParam = destination of the connection given by: "fbname.intput"
 *    - m_sAdditionalParams not used
 */
const TMGMCommandType cg_nMGM_CMD_Delete_Connection = 0x41;
/*! \brief Delete all FBs or resource instances.
 * 
 * When deleting a FB instance the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for deleteing within the device or "resname" for deleting within a resource
 *    - m_sFistParam not used
 *    - m_sSecondParam not used
 *    - m_sAdditionalParams not used
 */
const TMGMCommandType cg_nMGM_CMD_Delete_AllFBInstances = 0x61;
/*! \brief start a FB, resource or the device.
 * 
 * When starting a FB, resource or the device the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for starting within the device or "resname" for starting within a resource
 *    - m_sFirstParam = target to be started can be empty for starting the whole device or resource, or contain a name for the resource or FB to start.
 *    - m_sSecondParam = not used
 *    - m_sAdditionalParams not used
 */
const TMGMCommandType cg_nMGM_CMD_Start = 0x02;
/*! \brief stop a FB, resource or the device.
 * 
 * When stoping a FB, resource or the device the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for stoping within the device or "resname" for stoping within a resource
 *    - m_sFirstParam = target to be stoped can be empty for starting the whole device or resource, or contain a name for the resource or FB to stop.
 *    - m_sSecondParam = not used
 *    - m_sAdditionalParams not used
 */
const TMGMCommandType cg_nMGM_CMD_Stop = 0x03;
/*! \brief Read values from FB in- and outputs or from resource inputs
 * 
 * When reading a parameter value the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for creating within the device or "resname" for creating within a resource
 *    - m_sFistParam = output qualifier for reading outputs ("fbname.output") or "*" for reading inputs
 *    - m_sSecondParam = input qualifier for reading inputs ("fbname.input") or "*" for reading outputs
 *    - m_sAdditionalParams the read value is stored here
 */
const TMGMCommandType cg_nMGM_CMD_Read = 0x04;

/*! \brief Write value to a given data input or output (FB or Resource)
 * 
 * When writing a parameter value the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for creating within the device or "resname" for creating within a resource
 *    - m_sFistParam = "*" or source of the connection given by: "fbname.input"
 *    - m_sSecondParam = "*" or destination of the connection given by: "fbname.input"
 *    - m_sAdditionalParams = the string converted value to be set
 */
const TMGMCommandType cg_nMGM_CMD_Write = 0x05;
/*! \brief kill a FB, resource or the device.
 * 
 * When killing a FB, resource or the device the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for killing within the device or "resname" for stoping within a resource
 *    - m_sFistParam = target to be killed can be empty for starting the whole device or resource, or contain a name for the resource or FB to stop.
 *    - m_sSecondParam = not used
 *    - m_sAdditionalParams not used
 */
const TMGMCommandType cg_nMGM_CMD_Kill = 0x06;
/*! \brief query FBs
 * 
 * The parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" to query within the device or "resname" for fetching within a resource
 *    - m_acFirstParam = * for all FBs or instancename 
 *    - m_sSecondParam = * for all FB types or type name
 *    - m_sAdditionalParams  the response value of the query
 */
const TMGMCommandType cg_nMGM_CMD_QUERY_FB = 0x17;
/*! \brief query connections
 * 
 * The parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" to query within the device or "resname" for fetching within a resource
 *    - m_acFirstParam = *  
 *    - m_sSecondParam = * 
 *    - m_sAdditionalParams  the response value of the query - a list of event and data connections (comma separated list of end points to end points)
 */
const TMGMCommandType cg_nMGM_CMD_QUERY_Connection = 0x27;
/*! \brief Read the existing fb types.
 *  
 * When reading the fb types the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for reading within the device or "resname" for reading within a resource
 *    - m_sFistParam = not used
 *    - m_sSecondParam = not used
 *    - m_sAdditionalParams the read value is stored here
 */
const TMGMCommandType cg_nMGM_CMD_QUERY_FBTypes = 0x37;
/*! \brief Read the existing data types.
 *  
 * When reading the data types the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for reading within the device or "resname" for reading within a resource
 *    - m_sFistParam = not used
 *    - m_sSecondParam = not used
 *    - m_sAdditionalParams the read value is stored here
 */
const TMGMCommandType cg_nMGM_CMD_QUERY_DTTypes = 0x47;
/*! \brief Read the existing resource types.
 *  
 * When reading the resource types the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for reading within the device or "resname" for reading within a resource
 *    - m_sFistParam = not used
 *    - m_sSecondParam = not used
 *    - m_sAdditionalParams the read value is stored here
 */
const TMGMCommandType cg_nMGM_CMD_QUERY_RESTypes = 0x57;
/**
 * currently not implemented
 */
const TMGMCommandType cg_nMGM_CMD_QUERY_TypeVers = 0x67;
/*! \brief reset a FB, resource or the device.
 * 
 * When reseting a FB, resource or the device the parameters of the SManagementCMD are defined as:
 *    - m_sDestination = "" for reseting within the device or "resname" for reseting within a resource
 *    - m_sFistParam = target to be reset can be empty for reseting the whole device or resource, or contain a name for the resource or FB to reset.
 *    - m_sSecondParam = not used
 *    - m_sAdditionalParams not used
 */
const TMGMCommandType cg_nMGM_CMD_Reset = 0x08;
/*! \brief invalid command: some of the data could not be parsed
 */
const TMGMCommandType cg_nMGM_CMD_INVALID = 0xFF;

/*@}*///end von defgroup SuportedMGMCommands 

/*!\brief Type for the response of MGM commands
 * 
 * TODO fully define all responses as defined in IEC 61499 inc. numbers.
 */
enum EMGMResponse {e_RDY, e_BAD_PARAMS, e_LOCAL_TERMINATION, e_SYSTEM_TERMINATION, e_NOT_READY, e_UNSUPPORTED_CMD,
  e_UNSUPPORTED_TYPE, e_NO_SUCH_OBJECT, e_INVALID_OBJECT, e_INVALID_OPERATION, e_INVALID_STATE,
  e_OVERFLOW, e_INVALID_DST};

/*@}*/
#endif /*MGMCMD_H_*/
