/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20071029: az, alil@users.sourceforge.net - enhanced clean-up of connection list and FB list in destructor
 *******************************************************************************/
#ifndef _OBJHAND_H_
#define _OBJHAND_H_

#include "funcbloc.h"
#include <map>

/**\ingroup CORE
 *Class handling all managment commands and handling created objects like, FBs, Resources, Connections,...
 *
 *  @author Alois Zoitl <zoitl@acin.tuwien.ac.at>
 *  @version 20081121/mmm - changed due to elimination of ANY_FACADE
 *  @version 20071029/az - enhanced clean-up of connection list and FB list in destructor
*/
class C61499ObjectHandler : public CFunctionBlock{
  private:
  protected:
/*! \brief A reference to the typelib containing the available IEC 61499 types.
 *
 */
  CTypeLib &m_roTypeLib;

  typedef std::map<CStringDictionary::TStringId, CFunctionBlock *> TObjInstanceList;
/*! \brief List of the current instanciated FBs in this 61499 Object Handler (may also contain Resources).
 *
 */
  TObjInstanceList m_lstpoObjectInstanceList;

/*! \brief Type for storing the list of connection.
 *
 * The identifier is a combined string id for the source, containg the fb and the output. See CConnection for more details.
 */   
  typedef std::map<TConnectionID, CConnection *> TConnectionList;   
  
    
/*! \brief List of the current connections in this 61499 Object Handler.
 *
 *  This list contains the data, event, and adapter connections.
 */
  TConnectionList m_lstpoConnectionList;

/*!\brief get a pointer to the FBInstance with the given name
 *
 * \param pa_nFBInstanceNameId name Id of the FB to get
 * \return pointer to the FBInstance or 0 if such a FB does not exist within the resource.
 */
    CFunctionBlock *getFB(CStringDictionary::TStringId pa_nFBInstanceNameId);

/*!\brief check if there exists an conneciton with given source identifier in this resoruce
 *
 * \param pa_sSource source identifier of conection (FB.Output)
 * \return pointer to connection if connection does not exist 0
 */
    CConnection *getConnection(TConnectionID pa_nSourceId);

/** \brief Abstract function that serves as factory method for the FB Ojbects that are managed by this
  *  61499 Object handler.
  *
  * It has to be provided by derived classes and allows that different types like resource or special fbs
  * can be managed in this object.
  *
  * @param pa_nFBNameId instance name of the object to create
  * @param pa_nFBTypeId typename of the object to create
  * @param pa_nApplId the name of the application to which the object belongs to
  * @return on success a pointer to the created object or 0 on failure
  */
    virtual CFunctionBlock *createFBObject(CStringDictionary::TStringId pa_nInstanceNameId, CStringDictionary::TStringId pa_nResTypeId, CStringDictionary::TStringId pa_nApplId) = 0;

/** \brief Abstract function that handles the deletion for the FB Objects.
  *
  * @param pa_poFBObj pointer to the object to delete
  * @return true on success
  */
    virtual bool deleteFBObject(CFunctionBlock *pa_poFBObj) = 0;

/** \brief Delete all FB instances
  *
  * @return true on success
  */
    EMGMResponse deleteAllFB();

public:
    C61499ObjectHandler(CStringDictionary::TStringId pa_nAppl, CResource* pa_poSrcRes,
                        const SFBInterfaceSpec* pa_pstInterfaceSpec,
                        TEventConnectionPtr* pa_apoEOConns,
                        TDataConnectionPtr* pa_apoDIConns, TIEC_ANYPtr* pa_apoDIs,
                        TDataConnectionPtr* pa_apoDOConns, TIEC_ANYPtr* pa_apoDOs):
                       CFunctionBlock(pa_nAppl, pa_poSrcRes, pa_pstInterfaceSpec,
                                      pa_apoEOConns,
                                      pa_apoDIConns, pa_apoDIs,
                                      pa_apoDOConns, pa_apoDOs),
                      m_roTypeLib(CTypeLib::getInstance()){
    };

    virtual ~C61499ObjectHandler(){
      deleteAllFB();
    };

/*!\brief Execute the given management command
 *
 * Evaluates the m_sDestination parameter of the command if empty the resource tries
 * to execute the management command if not it tries to apply the mgm command to a
 * contained FB if possible.
 * \param pa_oCommand internal representation of the management command
 * \return response of the MGMCommand execution as defined in IEC 61499
 */
    virtual EMGMResponse executeMGMCommand(SManagementCMD &pa_oCommand);

/*!\brief Create a new FB instance of given type and name
 *
 * @param pa_acFBName instance name of the new FB
 * @param pa_acFBType type name of the new FB
 * @param sAppl the name of the application to which the FB belongs to
 * @return response of the command execution as defined in IEC 61499
 */
    EMGMResponse createFB(CStringDictionary::TStringId pa_nFBNameId, CStringDictionary::TStringId pa_nFBTypeId, const char *pa_acAppl);

/*!\brief Create a new connection between source and destination
 *
 * @param pa_acSource source of the connection fb.output
 * @param pa_acDest destination of the connection fb.input
 * @return response of the command execution as defined in IEC 61499
 */
    EMGMResponse createConnection(TConnectionID pa_nSourceId, TConnectionID pa_nDestId);

/*!\brief Delete a FB instance with given name
 *
 * @param pa_acFBName instance name of the new FB
 * @return response of the command execution as defined in IEC 61499
 */
    EMGMResponse deleteFB(CStringDictionary::TStringId pa_nFBNameId);

/*!\brief Create a new connection between source and destination
 *
 * @param pa_acSource source of the connection fb.output
 * @param pa_acDest destination of the connection fb.input
 * @return response of the command execution as defined in IEC 61499
 */
    EMGMResponse deleteConnection(TConnectionID pa_nSourceId, TConnectionID pa_nDestId);

    EMGMResponse createFBType(TIdentifier &pa_acFBName);

/*!\brief Write a parameter value to a given FB-input
 *
 * @param pa_acDest FB value qualifier or (FBName.DataPortName), may be inpupt, output, or internal var
 * @param pa_acValue String converted value of the parameter to value
 * @return response of the command execution as defined in IEC 61499
 */
    EMGMResponse writeValue(TConnectionID pa_nDestId, const char *pa_acValue);


/*!\brief Read a parameter value from a given FB
 *
 * @param pa_nSourceId FB value qualifier or (FBName.DataPortName), may be inpupt, output, or internal var
 * @param pa_sValue the value that has been read back
 * @return response of the command execution as defined in IEC 61499
 */
    EMGMResponse readValue(TConnectionID pa_nSourceId, string &pa_sValue);

    EMGMResponse queryFBTypes(string &pa_sValue);
    EMGMResponse queryRESTypes(string &pa_sValue);
    EMGMResponse queryDTTypes(string &pa_sValue);
    EMGMResponse queryTypeVersion(TUINT32 pa_nSource, string &pa_sValue);
/*!\brief Read a parameter value from a given FB
 *
 * @param pa_acSource fb instancename or * (to check if it exists)
 * @param pa_acDest FB type qualifier or * (all fb instances of the defined type if not *)
 * @param pa_sResponse the value of the existing instancenames (comma separated list)
 * @return response of the command execution as defined in IEC 61499
 */    
    EMGMResponse executeQueryReq(TConnectionID pa_nSourceId, TConnectionID &pa_nDestId, string &pa_sResponse);   

 /*! \brief query connections
 *
 * The parameters of the SManagementCMD are defined as:
 * @param pa_acSource = *
 * @param pa_acDest = *
 * @param pa_sResponse the result of the query - a list of event and data connections (comma separated list of end points to end points)
 * @return response of the command execution as defined in IEC 61499
 */
    EMGMResponse executeQueryCon(TConnectionID pa_nSourceId, TConnectionID pa_nDestId, string &pa_sResponse);

/*! \brief start the whole resource or the given FB
 *
 * @param pa_acFBName nstance name of the FB to start if an empty string is
 *        given all FBs in the resource will be started.
 * @return response of the command execution as defined in IEC 61499
 */
    EMGMResponse executeStartReq(CStringDictionary::TStringId pa_nFBNameId);

/*! \brief stop the wholeresource or the given FB
 *
 * @param pa_acFBName nstance name of the FB to stop if an empty string is
 *        given all FBs in the resource will be stoped.
 * @return response of the command execution as defined in IEC 61499
 */
    EMGMResponse executeStopReq(CStringDictionary::TStringId pa_nFBNameId);

/*! \brief kill the whole resource or the given FB
 *
 * @param pa_acFBName nstance name of the FB to stop if an empty string is
 *        given all FBs in the resource will be stoped.
 * @return response of the command execution as defined in IEC 61499
 */
    EMGMResponse executeKillReq(CStringDictionary::TStringId pa_nFBNameId);

/*! \brief reset the whole resource or the given FB
 *
 * @param pa_acFBName instance name of the FB to reset if an empty string is
 *        given all FBs in the resource will be reset.
 * @return response of the command execution as defined in IEC 61499
 */
    EMGMResponse executeResetReq(CStringDictionary::TStringId pa_nFBNameId);

 // bool query(...);  //will be intersting

/*! \brief Start the runable object.
 *
 * Derived objects will add aditional functionality to this function (e.g. E_RESTART)
 * @return response of the command execution as defined in IEC 61499
 */
    virtual EMGMResponse startManagedObject(void);

/*! \brief Stop the runable object.
 *
 * Derived objects will add aditional functionality to this function (e.g. E_RESTART)
 * @return response of the command execution as defined in IEC 61499
 */
    virtual EMGMResponse stopManagedObject(void);

/*! \brief Kill the FB.
 *
 * Derived objects will add aditional functionality to this function (e.g. E_RESTART)
 * @return response of the command execution as defined in IEC 61499
 */
    virtual EMGMResponse killManagedObject(void);

/*!\brief Reset the runable object.
 *
 * Derived objects will add aditional functionality to this function (e.g. E_RESTART)
 * @return response of the command execution as defined in IEC 61499
 */
    virtual EMGMResponse resetManagedObject(void);


};

#endif
