/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20081216: mmm - initial version
 *******************************************************************************/

#ifndef CITASERDESER_H_
#define CITASERDESER_H_

#include "ISerDeser.h"
#include "../../arch/timerha.h"
#include <string>

/* \brief Serializer/Deserializer class according to the IEC 61499 Compliance Profile for
 *   Feasibility Demonstrations based on ISO/IEC 8825 (ASN.1).
 *   http://www.holobloc.com/doc/ita/index.htm Chapter 5.4 Presentation Layer
 */

class CITASerDeser: public ISerDeser {
  SINGLETON(CITASerDeser)
public:
  virtual ~CITASerDeser();
  virtual int serialize(TBYTE* pa_pcBytes, int pa_nStreamSize, TIEC_ANYPtr pa_pCIECData);
  virtual int deserialize(TBYTE* pa_pcBytes, int pa_nStreamSize, TIEC_ANYPtr pa_pCIECData);
  virtual void serializeNull(TBYTE* pa_pcBytes)  {
    *pa_pcBytes = scm_nNull;
  }
  bool isNull(TBYTE* pa_pcBytes) {
    return *pa_pcBytes == scm_nNull;
  }

protected:
  static const TBYTE scm_nNull = 5;
  enum EDataTypeTags  {
    e_ANY_TAG                 = 0,
    e_BOOL_TAG                = 1,
    e_SINT_TAG                = 2,
    e_INT_TAG                 = 3,
    e_DINT_TAG                = 4,
    e_LINT_TAG                = 5,
    e_USINT_TAG               = 6,
    e_UINT_TAG                = 7,
    e_UDINT_TAG               = 8,
    e_ULINT_TAG               = 9,
    e_REAL_TAG                = 10,
    e_LREAL_TAG               = 11,
    e_TIME_TAG                = 12,
    e_DATE_TAG                = 13,
    e_TIME_OF_DAY_TAG         = 14,
    e_DATE_AND_TIME_TAG       = 15,
    e_STRING_TAG              = 16,
    e_BYTE_TAG                = 17,
    e_WORD_TAG                = 18,
    e_DWORD_TAG               = 19,
    e_LWORD_TAG               = 20,
    e_WSTRING_TAG             = 21,
    e_DerivedData_TAG         = 26,
    e_DirectlyDerivedData_TAG = 27,
    e_EnumeratedData_TAG      = 28,
    e_SubrangeData_TAG        = 29,
    e_ARRAY_TAG               = 22,  //according to the compliance profile
    e_STRUCT_TAG              = 31
  };
  /*! \ingroup COREDTS\brief ASN.1 tag classes
   */
  enum EASN1TagClass {
    e_UNIVERSAL           = 0,
    e_APPLICATION         = 64,
    e_CONTEXT             = 128,
    e_PRIVATE             = 192
  };
  /*! \ingroup COREDTS\brief ASN.1 encoding types
   */
  enum EASN1EncodingType {
    e_PRIMITIVE           = 0,
    e_CONSTRUCTED         = 32
  };

  const static TBYTE csm_aDataTags[][2];

private:

  /*! \brief Serialization of data tag  according to IEC 61499 Compliance Profile for
   *   Feasibility Demonstrations based on ISO/IEC 8825 (ASN.1).
   *
   *   This command implements the serialize for the ASN TAG defining the data type.
   *   Pure virtual function implementation.
   *   \param pa_pcBytes  Array to write the serialized TAG to
   *   \return Can be the following response:
   *     - retval > 0....number of bytes the data value will need in the serialized array
   *     - retval =-1....error
   */
  inline int serializeTag(TBYTE* pa_pcBytes, TIEC_ANYPtr pa_pCIECData)  {
    pa_pcBytes[0] = csm_aDataTags[pa_pCIECData->getDataTypeID()][0];
    switch(pa_pCIECData->getDataTypeID()) {
      case CIEC_ANY::e_STRING:  {
        return ((CIEC_STRING*)pa_pCIECData)->getValue().length();
      }
      case CIEC_ANY::e_WSTRING:  {
        return ((CIEC_WSTRING*)pa_pCIECData)->getValue().length();
      }
      case CIEC_ANY::e_ARRAY:  {
        return ((CIEC_ARRAY*)pa_pCIECData)->size();
      }
      default:  {
        return csm_aDataTags[pa_pCIECData->getDataTypeID()][1];
      }
    }

  }

  /*! \brief Deserialization of data tag  according to IEC 61499 Compliance Profile for
   *   Feasibility Demonstrations based on ISO/IEC 8825 (ASN.1).
   *
   *   Pure virtual function implementation.
   *   \param pa_cByte byte that contains the received tag.
   *   \return Can be the following response:
   *     - retval = 1....the Tag is ok
   *     - retval =-1....error
   */
  inline int deserializeTag(TBYTE* pa_pcBytes, TIEC_ANYPtr pa_pCIECData)  {
    CIEC_ANY::EDataTypeID eDataType = pa_pCIECData->getDataTypeID();
    if(eDataType == CIEC_ANY::e_BOOL)
    {
      return (((e_APPLICATION + e_PRIMITIVE) == pa_pcBytes[0] || (e_APPLICATION + e_PRIMITIVE + CIEC_ANY::e_BOOL) == pa_pcBytes[0]) ? 1 : -1);
    }
    return (csm_aDataTags[eDataType][0] == pa_pcBytes[0] ) ? 1 : -1;
  }

  /*! \brief Serialization of the data value according to IEC 61499 Compliance Profile for
   *   Feasibility Demonstrations based on ISO/IEC 8825 (ASN.1).
   *
   *   This command implements the serialize for the data of the data type. No check on the size needs to be performed as this is hanlded in the serilze function of CIEC_ANY.
   *   Pure virtual function implementation.
   *   \param pa_pcBytes  Array to write the serialized data to
   *   \return Can be the following response:
   *     - retval = 1....serialization went well
   *     - retval =-1....error
   */
  int serializeValue(TBYTE* pa_pcBytes, int pa_nStreamSize, TIEC_ANYPtr pa_pCIECData, unsigned int nSize);

  /*! \brief Deerialization of the data value according to IEC 61499 Compliance Profile for
   *   Feasibility Demonstrations based on ISO/IEC 8825 (ASN.1).
   *
   *   This command implements the deserialize for the data of the data type. No check on the size needs to be performed as this is hanlded in the deserialize function of CIEC_ANY.
   *   Pure virtual function implementation.
   *   \param pa_pcBytes  Array to read the data from to
   *   \return Can be the following response:
   *     - retval > 0....deserialization went well, number of bytes used for deserializing the value
   *     - retval =-1....error
   */
  int deserializeValue(TBYTE* pa_pcBytes, int pa_nStreamSize, TIEC_ANYPtr pa_pCIECData);
};

#endif /* CITASERDESER_H_ */
