/*******************************************************************************
 * Copyright (c) 2008 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/

#include "stringdict.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

const CStringDictionary::TStringId CStringDictionary::scm_nInvalidStringId = 0xffffffff;

// ctor
CStringDictionary::CStringDictionary()
: m_paBufAddr(0), m_nBufSize(0), m_nMaxNrOfStrings(0), m_nNrOfStrings(0), m_nNextString(0), m_bOurMgmt(false) {
  init(8000, 2048);
}

// Initialize with external address and size. Won't realloc, you delete
bool CStringDictionary::init(void *pa_pBufAddr, unsigned int pa_nBufSize, unsigned int pa_nNrOfStrings) {
	if (m_paBufAddr != 0 || pa_nBufSize < 1024 || pa_nNrOfStrings*sizeof(TStringId) >= pa_nBufSize)
		return false;

	m_paBufAddr = (char *) pa_pBufAddr;
	m_nBufSize = pa_nBufSize;
	m_nMaxNrOfStrings = pa_nNrOfStrings;

	m_nNrOfStrings = 0;
	m_nNextString = 0;

	m_bOurMgmt = false;

	return true;
}

// Initialize with internal memory management
bool CStringDictionary::init(unsigned int pa_nInitialSizeForStrings, unsigned int pa_nInitialNrOfStrings) {
	if (m_paBufAddr != 0)
		return false;

	unsigned int allocSize = getAllocSize(pa_nInitialSizeForStrings, pa_nInitialNrOfStrings);
	m_paBufAddr = (char *) malloc(allocSize);
	if (m_paBufAddr == 0)
		return false;

	m_nBufSize = allocSize;
	m_nMaxNrOfStrings = pa_nInitialNrOfStrings;

	m_nNrOfStrings = 0;
	m_nNextString = 0;

	m_bOurMgmt = true;

	return true;
}

// clear
void CStringDictionary::clear() {
	if (m_bOurMgmt && m_paBufAddr != 0)
		free(m_paBufAddr);

	m_paBufAddr = 0;
	m_nBufSize = 0;
	m_nMaxNrOfStrings = 0;
	m_nNrOfStrings = 0;
	m_nNextString = 0;
	m_bOurMgmt = false;
}

// get a string (0 if not found)
const char *CStringDictionary::get(TStringId pa_nId) {
	if (pa_nId >= m_nNextString)
		return 0;

	const char *adr = getStringAddress(pa_nId);
	if (pa_nId > 0 && adr[-1] != '\0')
		return 0;

	return adr;
}

// insert a string and return a string id (InvalidTStringId for no memory or other error)
CStringDictionary::TStringId CStringDictionary::insert(const char *pa_sStr) {
	if (! pa_sStr)
		return scm_nInvalidStringId;

	unsigned int len = (unsigned int) strlen(pa_sStr);
	unsigned int newSize = (unsigned int) (getStringAddress(m_nNextString) + len + 1 - m_paBufAddr);

	if (newSize > m_nBufSize || m_nNrOfStrings >= m_nMaxNrOfStrings) {
		unsigned int newNrOfStrings = m_nNrOfStrings >= m_nMaxNrOfStrings ? m_nMaxNrOfStrings * 3 / 2 : m_nMaxNrOfStrings;
		newSize += (newNrOfStrings - m_nMaxNrOfStrings) * sizeof(TStringId);
		if (! m_bOurMgmt ||
			  ! reallocate(newSize > m_nBufSize ? newSize : m_nBufSize, newNrOfStrings))
			return scm_nInvalidStringId;
	}
	unsigned int idx;
	bool exactMatch;

	TStringId id = findEntry(pa_sStr, idx, exactMatch);
	if (exactMatch)
		return id;

	id = insertAt(pa_sStr, idx, len);

	return id;
}

// Find an exact match or place to be the new index
CStringDictionary::TStringId CStringDictionary::findEntry(const char *pa_sStr, unsigned int &pa_rnIdx, bool &pa_rbExactMatch) const {
	pa_rnIdx = 0;
	pa_rbExactMatch = false;

	if (m_nNrOfStrings == 0)
		return scm_nInvalidStringId;

	const TStringId *sp = (TStringId *) m_paBufAddr;
	int r = 0;

	unsigned int low = 0, high = m_nNrOfStrings;

	while (low < high) {
		pa_rnIdx = (low + high) / 2;

		r = strcmp(pa_sStr, getStringAddress(sp[pa_rnIdx]));

		if (! r) {
			pa_rbExactMatch = true;
			return sp[pa_rnIdx];
		}

		if (r > 0)
			low = pa_rnIdx+1;
		else
			high = pa_rnIdx;
	}

	if (r > 0)
		pa_rnIdx++;

	return scm_nInvalidStringId;
}

// Reallocate the buffer
bool CStringDictionary::reallocate(unsigned int pa_nNewBufSize, unsigned int pa_nNewNrOfStrings)
{
	if (pa_nNewBufSize <= m_nBufSize && pa_nNewNrOfStrings <= m_nMaxNrOfStrings)
		return true;

	if (pa_nNewBufSize <= m_nBufSize)
		pa_nNewBufSize = m_nBufSize;

	if (pa_nNewNrOfStrings <= m_nMaxNrOfStrings)
		pa_nNewNrOfStrings = m_nMaxNrOfStrings;

	unsigned int allocSize = getAllocSize(pa_nNewBufSize - pa_nNewNrOfStrings*sizeof(TStringId), pa_nNewNrOfStrings);

	char *adr = (char *) realloc(m_paBufAddr, allocSize);
	if (adr == 0)
		return false;

	m_paBufAddr = adr;
	m_nBufSize = allocSize;

	if (pa_nNewNrOfStrings > m_nMaxNrOfStrings && m_nNextString > 0)
		memmove(m_paBufAddr + pa_nNewNrOfStrings*sizeof(TStringId), m_paBufAddr + m_nMaxNrOfStrings*sizeof(TStringId), m_nNextString);

	m_nMaxNrOfStrings = pa_nNewNrOfStrings;

	return true;
}

// Insert the string at the specified position
CStringDictionary::TStringId CStringDictionary::insertAt(const char *pa_sStr, unsigned int pa_nIdx, unsigned int pa_nLen)
{
	TStringId id = m_nNextString;
	char *p = getStringAddress(m_nNextString);

	memcpy(p, pa_sStr, pa_nLen);
	p[pa_nLen] = '\0';
	m_nNextString += pa_nLen+1;

	TStringId *sp = (TStringId *) m_paBufAddr;

	if (pa_nIdx < m_nNrOfStrings)
		memmove(sp+pa_nIdx+1, sp+pa_nIdx, (m_nNrOfStrings-pa_nIdx) * sizeof(TStringId));

	sp[pa_nIdx] = id;
	m_nNrOfStrings++;

	return id;
}

// Suggest an allocation size to allocate something near power of two boundary
unsigned int CStringDictionary::getAllocSize(unsigned int pa_nSizeForStrings, unsigned int pa_nNrOfStrings) const
{
	unsigned int needed = pa_nSizeForStrings + pa_nNrOfStrings*sizeof(TStringId) + 32;
	unsigned x = 1024;
	for (unsigned int i=0; i < 10; ++i, x <<= 1) {
		if (x >= needed)
			break;
	}

	if (x < needed)
		return 0;

	return x-32;
}
