/*******************************************************************************
 * Copyright (c) 2008 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/

#ifndef _CStringDictionary_H_
#define _CStringDictionary_H_

#include "singlet.h"
#include "../arch/datatype.h"

/**\ingroup CORE\brief Manages a dictionary of strings that can be referenced by ids

	Manages a dictionary of strings that can be referenced by ids

	@author Stanislav Meduna
*/
class CStringDictionary{
  SINGLETON(CStringDictionary)
public:
	typedef TUINT32 TStringId;

	//!\brief Invalid string id
	static const TStringId scm_nInvalidStringId;

	~CStringDictionary(){};

/*! \brief Initialize with external address and size. Won't realloc, you are responsible for delete
 * 
 * \param pa_pBufAddr Provided buffer
 * \param pa_nBufSize   Size of the provided buffer
 * \param pa_nNrOfStrings Number of strings the dictionary can hold
 * \return true on success
 */  
	bool init(void *pa_pBufAddr, unsigned int pa_nBufSize, unsigned int pa_nNrOfStrings);

/*!\brief Initialize with internal memory management, will realloc as needed
 * 
 * \param pa_nInitialSizeForStrings Initial size to be allocated for strings
 * \param pa_nInitialNrOfStrings Initial number of strings the dictionary can hold
 * \return true on success
 */  
	bool init(unsigned int pa_nInitialSizeForStrings, unsigned int pa_nInitialNrOfStrings);

	//!\brief Remove all dictionary entries
	void clear();

/*!\brief Get a string from the dictionary
 * 
 * \param pa_nId String id
 * \return true the found string or 0 if not found
 */  
	const char *get(TStringId pa_nId);

/*!\brief Insert a string into the dictionary
 * 
 * \param pa_sStr String to be inserted
 * \return id of the inserted string (or the existing one if the dictionary already has it)
 */  
	TStringId insert(const char *pa_sStr);

/*!\brief Retreive the Id of a given string if it is already in the dictionary
 * 
 * \param pa_sStr String to be searched for
 * \return id of the string (or scm_nInvalidStringId if it is not in the dictionary)
 */  
	TStringId getId(const char *pa_sStr) const{
	  bool bExactMatch;
	  unsigned int nIdx;
	  return findEntry(pa_sStr, nIdx, bExactMatch);
	}
private:
  // Find an exact match or place to be the new index
  TStringId findEntry(const char *pa_sStr, unsigned int &pa_rnIdx, bool &pa_rbExactMatch) const;

  // Reallocate the buffer
  bool reallocate(unsigned int pa_nNewBufSize, unsigned int pa_nNewNrOfStrings);

  // Insert the string at the specified position
  TStringId insertAt(const char *pa_sStr, unsigned int pa_nIdx, unsigned int pa_nLen);

  // Get an address
  const char *getStringAddress(TStringId pa_nId) const { return m_paBufAddr + (m_nMaxNrOfStrings * sizeof(TStringId)) + pa_nId; }
  char *getStringAddress(TStringId pa_nId) { return m_paBufAddr + (m_nMaxNrOfStrings * sizeof(TStringId)) + pa_nId; }

  // Suggest an allocation size to allocate something near power of two boundary
  unsigned int getAllocSize(unsigned int pa_nSizeForStrings, unsigned int pa_nNrOfStrings) const;

  // Buffer address
  char *m_paBufAddr;

  // Size of the allocated space
  unsigned int m_nBufSize;

  // Maximum number of strings we can hold
  unsigned int m_nMaxNrOfStrings;

  // Number of strings we are actually holding
  unsigned int m_nNrOfStrings;

  // Next string gets written here
  TStringId m_nNextString;

  // Who is managing the buffer
  bool m_bOurMgmt;
};


#endif // _CStringDictionary_H_

